@@ -169,6 +169,7 @@ class DecoratorFinder(ast.NodeVisitor):
             self.decorators[node.name].append(name)
 
 
+@unittest.skipUnless(NumpyDocString, "requires 'numpydoc', install openmdao[test]")
 class LintTestCase(unittest.TestCase):
 
     def check_summary(self, numpy_doc_string):
@@ -177,6 +178,9 @@ class LintTestCase(unittest.TestCase):
         ----------
         numpy_doc_string : numpydoc.docscrape.NumpyDocString
             An instance of the NumpyDocString parsed from the method
+
+        Returns
+        -------
         failures : dict
             The failures encountered by the method.  These are all stored
             so that we can fail once at the end of the check_method method
@@ -204,17 +208,20 @@ class LintTestCase(unittest.TestCase):
 
         return new_failures
 
-    def check_parameters(self, func, argspec, numpy_doc_string):
-        """ Check that the parameters section is correct.
+    def check_parameters(self, argspec, numpy_doc_string):
+        """
+        Check that the parameters section is correct.
 
         Parameters
         ----------
-        func :
         argspec : namedtuple
             Method argument information from inspect.getargspec (python2) or
             inspect.getfullargspec (python3)
         numpy_doc_string : numpydoc.docscrape.NumpyDocString
             An instance of the NumpyDocString parsed from the method
+
+        Returns
+        -------
         failures : dict
             The failures encountered by the method.  These are all stored
             so that we can fail once at the end of the check_method method
@@ -280,8 +287,9 @@ class LintTestCase(unittest.TestCase):
 
         return new_failures
 
-    def check_returns(self, func, numpy_doc_string):
-        """ Check that the returns section is correct.
+    def check_returns(self, func, numpy_doc_string, name_required=False):
+        """
+        Check that the returns section is correct.
 
         Parameters
         ----------
@@ -289,6 +297,11 @@ class LintTestCase(unittest.TestCase):
             The method being checked
         numpy_doc_string : numpydoc.docscrape.NumpyDocString
             An instance of the NumpyDocString parsed from the method
+        name_required : bool
+            If True, a name is required for the return value.
+
+        Returns
+        -------
         failures : dict
             The failures encountered by the method.  These are all stored
             so that we can fail once at the end of the check_method method
@@ -301,7 +314,13 @@ class LintTestCase(unittest.TestCase):
         dedented_src = textwrap.dedent(method_src)
 
         f = ReturnFinder()
-        f.visit(ast.parse(dedented_src))
+        try:
+            f.visit(ast.parse(dedented_src))
+        except SyntaxError:
+            # ast.parse in python 2 will fail if we use certain print function syntax in
+            # our function.
+            dedented_src = 'from __future__ import print_function\n' + dedented_src
+            f.visit(ast.parse(dedented_src))
 
         # If the function does nothing but pass, return
         if f.passes:
@@ -322,10 +341,8 @@ class LintTestCase(unittest.TestCase):
                                 'no \'Returns\' section in docstring')
         elif f.has_return and doc_returns:
             # Check formatting
-            for entry in doc_returns:
-                name = entry[0]
-                desc = '\n'.join(entry[2])
-                if not name:
+            for (name, typ, desc) in doc_returns:
+                if name_required and not name:
                     new_failures.append('no detectable name for Return '
                                         'value'.format(name))
                 if desc == '':
@@ -334,7 +351,6 @@ class LintTestCase(unittest.TestCase):
 
         return new_failures
 
-    @unittest.skipUnless(NumpyDocString, "requires 'NumpyDocString', install openmdao[test]")
     def check_method(self, dir_name, file_name,
                      class_name, method_name, method, failures):
         """
@@ -385,7 +401,7 @@ class LintTestCase(unittest.TestCase):
 
         new_failures.extend(self.check_summary(nds))
 
-        new_failures.extend(self.check_parameters(method, argspec, nds))
+        new_failures.extend(self.check_parameters(argspec, nds))
 
         new_failures.extend(self.check_returns(method, nds))
 
@@ -398,6 +414,25 @@ class LintTestCase(unittest.TestCase):
                 failures[key] = new_failures
 
     def check_class(self, dir_name, file_name, class_name, clss, failures):
+        """
+        Perform docstring checks on a class.
+
+        Parameters
+        ----------
+        dir_name : str
+            The name of the directory in which the method is defined.
+        file_name : str
+            The name of the file in which the method is defined.
+        class_name : str
+            The name of the class being checked.
+        clss : class
+            The class being tested.
+        failures : dict
+            The failures encountered by the method.  These are all stored
+            so that we can fail once at the end of the check_method method
+            with information about every failure. Form is
+            { 'dir_name/file_name:class_name.method_name': [ messages ] }
+        """
 
         new_failures = []
         doc = inspect.getdoc(clss)
@@ -417,9 +452,9 @@ class LintTestCase(unittest.TestCase):
             else:
                 failures[key] = new_failures
 
-    @unittest.skipUnless(NumpyDocString, "requires 'NumpyDocString', install openmdao[test]")
     def check_function(self, dir_name, file_name, func_name, func, failures):
-        """ Perform docstring checks on a function.
+        """
+        Perform docstring checks on a function.
 
         Parameters
         ----------
@@ -429,7 +464,7 @@ class LintTestCase(unittest.TestCase):
             The name of the file in which the method is defined.
         func_name : str
             The name of the function being checked
-        fun : function
+        func : function
             The function being tested.
         failures : dict
             The failures encountered by the method.  These are all stored
@@ -466,7 +501,7 @@ class LintTestCase(unittest.TestCase):
 
         new_failures.extend(self.check_summary(nds))
 
-        new_failures.extend(self.check_parameters(func, argspec, nds))
+        new_failures.extend(self.check_parameters(argspec, nds))
 
         new_failures.extend(self.check_returns(func, nds))
 
