@@ -93,7 +93,7 @@ class AddSubtractComp(ExplicitComponent):
         complex : Boolean
             Set True to enable complex math (e.g. for complex step verification)
         """
-        self.options.declare('complex', default=False,
+        self.options.declare('complex', types=bool, default=False,
                              desc="Allocate as complex (e.g. for complex-step verification)")
 
     def add_equation(self, output_name, input_names, vec_size=1, length=1, val=1.0,
