@@ -2,6 +2,7 @@
 
 from __future__ import print_function, division, absolute_import
 
+from types import FunctionType
 from numbers import Number
 from six import iteritems
 
@@ -21,8 +22,16 @@ class BalanceComp(ImplicitComponent):
         so everything can be saved until setup is called.
     """
 
-    def __init__(self, name=None, eq_units=None, lhs_name=None,
-                 rhs_name=None, rhs_val=0.0, guess_func=None,
+    def initialize(self):
+        """
+        Declare options.
+        """
+        self.options.declare('guess_func', types=FunctionType, allow_none=True, default=None,
+                             desc='A callable function in the form f(inputs, outputs, residuals) '
+                                  'that can provide an initial "guess" value of the state '
+                                  'variable(s) based on the inputs, outputs and residuals.')
+
+    def __init__(self, name=None, eq_units=None, lhs_name=None, rhs_name=None, rhs_val=0.0,
                  use_mult=False, mult_name=None, mult_val=1.0, normalize=True, **kwargs):
         r"""
         Initialize a BalanceComp, optionally creating a new implicit state variable.
@@ -109,10 +118,6 @@ class BalanceComp(ImplicitComponent):
         rhs_val : int, float, or np.array
             Default value for the RHS of the given state.  Must be compatible
             with the shape (optionally) given by the val or shape option in kwargs.
-        guess_func : callable or None
-            A callable function in the form f(inputs, resids) that returns an initial "guess" value
-            of the state variable based on the inputs to the BalanceComp.  Note you may have to
-            add additional inputs to the BalanceComp in order to evaluate this function.
         use_mult : bool
             Specifies whether the LHS multiplier is to be used.  If True, then an additional
             input `mult_name` is created, with the default value given by `mult_val`, that
@@ -130,10 +135,16 @@ class BalanceComp(ImplicitComponent):
             Additional arguments to be passed for the creation of the implicit state variable.
             (see `add_output` method).
         """
-        super(BalanceComp, self).__init__()
+        if 'guess_func' in kwargs:
+            super(BalanceComp, self).__init__(guess_func=kwargs['guess_func'])
+            kwargs.pop('guess_func')
+        else:
+            super(BalanceComp, self).__init__()
+
         self._state_vars = {}
+
         if name is not None:
-            self.add_balance(name, eq_units, lhs_name, rhs_name, rhs_val, guess_func,
+            self.add_balance(name, eq_units, lhs_name, rhs_name, rhs_val,
                              use_mult, mult_name, mult_val, normalize, **kwargs)
 
     def setup(self):
@@ -142,19 +153,14 @@ class BalanceComp(ImplicitComponent):
         """
         for name, options in iteritems(self._state_vars):
 
+            meta = self.add_output(name, **options['kwargs'])
+
+            n = self._state_vars[name]['size'] = meta['size']
+
             for s in ('lhs', 'rhs', 'mult'):
                 if options['{0}_name'.format(s)] is None:
                     options['{0}_name'.format(s)] = '{0}:{1}'.format(s, name)
 
-            val = options['kwargs'].get('val', np.ones(1))
-            if isinstance(val, Number):
-                n = 1
-            else:
-                n = len(val)
-            self._state_vars[name]['size'] = n
-
-            self.add_output(name, **options['kwargs'])
-
             self.add_input(options['lhs_name'],
                            val=np.ones(n),
                            units=options['eq_units'])
@@ -191,6 +197,11 @@ class BalanceComp(ImplicitComponent):
         residuals : Vector
             unscaled, dimensional residuals written to via residuals[key]
         """
+        if inputs._under_complex_step:
+            self._scale_factor = self._scale_factor.astype(np.complex)
+        else:
+            self._scale_factor = self._scale_factor.real
+
         for name, options in iteritems(self._state_vars):
             lhs = inputs[options['lhs_name']]
             rhs = inputs[options['rhs_name']]
@@ -225,6 +236,11 @@ class BalanceComp(ImplicitComponent):
         jacobian : Jacobian
             sub-jac components written to jacobian[output_name, input_name]
         """
+        if inputs._under_complex_step:
+            self._dscale_drhs = self._dscale_drhs.astype(np.complex)
+        else:
+            self._dscale_drhs = self._dscale_drhs.real
+
         for name, options in iteritems(self._state_vars):
             lhs_name = options['lhs_name']
             rhs_name = options['rhs_name']
@@ -264,7 +280,9 @@ class BalanceComp(ImplicitComponent):
 
     def guess_nonlinear(self, inputs, outputs, residuals):
         """
-        Provide an "guess" for each output based on the values of the inputs and resids.
+        Provide initial guess for states.
+
+        Override this method to set the initial guess for states.
 
         Parameters
         ----------
@@ -275,12 +293,10 @@ class BalanceComp(ImplicitComponent):
         residuals : Vector
             unscaled, dimensional residuals written to via residuals[key]
         """
-        for name, options in iteritems(self._state_vars):
-            if options['guess_func'] is not None:
-                outputs[name] = options['guess_func'](inputs, residuals)
+        if self.options['guess_func'] is not None:
+            self.options['guess_func'](inputs, outputs, residuals)
 
-    def add_balance(self, name, eq_units=None, lhs_name=None,
-                    rhs_name=None, rhs_val=0.0, guess_func=None,
+    def add_balance(self, name, eq_units=None, lhs_name=None, rhs_name=None, rhs_val=0.0,
                     use_mult=False, mult_name=None, mult_val=1.0, normalize=True, **kwargs):
         """
         Add a new state variable and associated equation to be balanced.
@@ -304,10 +320,6 @@ class BalanceComp(ImplicitComponent):
         rhs_val : int, float, or np.array
             Default value for the RHS.  Must be compatible with the shape (optionally)
             given by the val or shape option in kwargs.
-        guess_func : callable or None
-            A callable function in the form f(inputs, resids) that returns an initial "guess" value
-            of the state variable based on the inputs to the BalanceComp.  Note you may have to
-            add additional inputs to the BalanceComp in order to evaluate this function.
         use_mult : bool
             Specifies whether the LHS multiplier is to be used.  If True, then an additional
             input `mult_name` is created, with the default value given by `mult_val`, that
@@ -325,15 +337,11 @@ class BalanceComp(ImplicitComponent):
             Additional arguments to be passed for the creation of the implicit state variable.
             (see `add_output` method).
         """
-        if guess_func is not None and not callable(guess_func):
-            raise ValueError("Argument 'guess_func' must be a callable if specified")
-
         self._state_vars[name] = {'kwargs': kwargs,
                                   'eq_units': eq_units,
                                   'lhs_name': lhs_name,
                                   'rhs_name': rhs_name,
                                   'rhs_val': rhs_val,
-                                  'guess_func': guess_func,
                                   'use_mult': use_mult,
                                   'mult_name': mult_name,
                                   'mult_val': mult_val,
