@@ -259,6 +259,14 @@ _ufunc_test_data = {
 
 class TestExecComp(unittest.TestCase):
 
+    def test_no_expr(self):
+        prob = Problem(model=Group())
+        prob.model.add_subsystem('C1', ExecComp())
+        with self.assertRaises(Exception) as context:
+            prob.setup(check=False)
+        self.assertEqual(str(context.exception),
+                         "C1: No valid expressions provided to ExecComp(): [].")
+
     def test_colon_vars(self):
         prob = Problem(model=Group())
         prob.model.add_subsystem('C1', ExecComp('y=foo:bar+1.'))
@@ -390,7 +398,9 @@ class TestExecComp(unittest.TestCase):
                                                     y={'units': 'm'},
                                                     units=2.0))
 
-        self.assertEqual(str(cm.exception), "The units argument should be a str or None.")
+        self.assertEqual(str(cm.exception),
+                         "Value (2.0) of option 'units' has type 'float', "
+                         "but type 'str' was expected.")
 
     def test_units_varname_str(self):
         prob = Problem(model=Group())
@@ -420,18 +430,31 @@ class TestExecComp(unittest.TestCase):
     def test_common_units(self):
         # all variables in the ExecComp have the same units
         prob = Problem(model=Group())
+
         prob.model.add_subsystem('indep', IndepVarComp('x', 100.0, units='cm'))
-        C1 = prob.model.add_subsystem('C1', ExecComp('y=x+z+1.', units='m',
-                                                     x={'value': 2.0},
-                                                     z=2.0))
-        prob.model.connect('indep.x', 'C1.x')
+        prob.model.add_subsystem('comp', ExecComp('y=x+z+1.', units='m',
+                                                  x={'value': 2.0},
+                                                  z=2.0))
+        prob.model.connect('indep.x', 'comp.x')
 
-        prob.setup(check=False)
-
-        prob.set_solver_print(level=0)
+        prob.setup()
         prob.run_model()
 
-        assert_rel_error(self, C1._outputs['y'], 4.0, 0.00001)
+        assert_rel_error(self, prob['comp.y'], 4.0, 0.00001)
+
+    def test_common_units_no_meta(self):
+        # make sure common units are assigned when no metadata is provided
+        prob = Problem()
+
+        prob.model.add_subsystem('indep', IndepVarComp('x', 2.0, units='km'))
+        prob.model.add_subsystem('comp', ExecComp('y = x+1', units='m'))
+
+        prob.model.connect('indep.x', 'comp.x')
+
+        prob.setup()
+        prob.run_model()
+
+        assert_rel_error(self, prob['comp.y'], 2001., 0.00001)
 
     def test_conflicting_units(self):
         prob = Problem(model=Group())
@@ -448,6 +471,110 @@ class TestExecComp(unittest.TestCase):
                          "C1: units of 'km' have been specified for variable 'x', but "
                          "units of 'm' have been specified for the entire component.")
 
+    def test_shape_and_value(self):
+        p = Problem()
+        model = p.model
+        model.add_subsystem('indep', IndepVarComp('x', val=np.ones(5)))
+
+        model.add_subsystem('comp', ExecComp('y=3.0*x + 2.5',
+                                             x={'shape': (5,), 'value': np.zeros(5)},
+                                             y={'shape': (5,), 'value': np.zeros(5)}))
+
+        model.connect('indep.x', 'comp.x')
+
+        p.setup()
+        p.run_model()
+
+        J = p.compute_totals(of=['comp.y'], wrt=['indep.x'], return_format='array')
+
+        assert_almost_equal(J, np.eye(5)*3., decimal=6)
+
+    def test_conflicting_shape(self):
+        p = Problem()
+        model = p.model
+        model.add_subsystem('indep', IndepVarComp('x', val=np.ones(5)))
+
+        model.add_subsystem('comp', ExecComp('y=3.0*x + 2.5',
+                                             x={'shape': (5,), 'value': 5},
+                                             y={'shape': (5,)}))
+
+        model.connect('indep.x', 'comp.x')
+
+        with self.assertRaises(Exception) as context:
+            p.setup()
+
+        self.assertEqual(str(context.exception).replace('L,', ','),  # L on Windows
+                         "comp: shape of (5,) has been specified for variable 'x', "
+                         "but a value of shape (1,) has been provided.")
+
+    def test_common_shape(self):
+        p = Problem()
+        model = p.model
+        model.add_subsystem('indep', IndepVarComp('x', val=np.ones(5)))
+
+        model.add_subsystem('comp', ExecComp('y=3.0*x + 2.5', shape=(5,)))
+
+        model.connect('indep.x', 'comp.x')
+
+        p.setup()
+        p.run_model()
+
+        J = p.compute_totals(of=['comp.y'], wrt=['indep.x'], return_format='array')
+
+        assert_almost_equal(J, np.eye(5)*3., decimal=6)
+
+    def test_common_shape_with_values(self):
+        p = Problem()
+        model = p.model
+        model.add_subsystem('indep', IndepVarComp('x', val=np.ones(5)))
+
+        model.add_subsystem('comp', ExecComp('y=3.0*x + 2.5', shape=(5,),
+                                             x={'value': np.zeros(5)},
+                                             y={'value': np.zeros(5)}))
+
+        model.connect('indep.x', 'comp.x')
+
+        p.setup()
+        p.run_model()
+
+        J = p.compute_totals(of=['comp.y'], wrt=['indep.x'], return_format='array')
+
+        assert_almost_equal(J, np.eye(5)*3., decimal=6)
+
+    def test_common_shape_conflicting_shape(self):
+        p = Problem()
+        model = p.model
+        model.add_subsystem('indep', IndepVarComp('x', val=np.ones(5)))
+
+        model.add_subsystem('comp', ExecComp('y=3.0*x + 2.5', shape=(5,),
+                                             y={'shape': (10,)}))
+
+        model.connect('indep.x', 'comp.x')
+
+        with self.assertRaises(Exception) as context:
+            p.setup()
+
+        self.assertEqual(str(context.exception).replace('L,', ','),  # L on Windows
+                         "comp: shape of (10,) has been specified for variable 'y', "
+                         "but shape of (5,) has been specified for the entire component.")
+
+    def test_common_shape_conflicting_value(self):
+        p = Problem()
+        model = p.model
+        model.add_subsystem('indep', IndepVarComp('x', val=np.ones(5)))
+
+        model.add_subsystem('comp', ExecComp('y=3.0*x + 2.5', shape=(5,),
+                                             x={'value': 5}))
+
+        model.connect('indep.x', 'comp.x')
+
+        with self.assertRaises(Exception) as context:
+            p.setup()
+
+        self.assertEqual(str(context.exception).replace('1L,', '1,'),  # 1L on Windows
+                         "comp: value of shape (1,) has been specified for variable 'x', "
+                         "but shape of (5,) has been specified for the entire component.")
+
     def test_math(self):
         prob = Problem(model=Group())
         C1 = prob.model.add_subsystem('C1', ExecComp('y=sin(x)', x=2.0))
@@ -517,7 +644,7 @@ class TestExecComp(unittest.TestCase):
         prob.set_solver_print(level=0)
         prob.run_model()
 
-        data = prob.check_partials()
+        data = prob.check_partials(out_stream=None)
 
         assert_rel_error(self, data['comp'][('y', 'x')]['abs error'][0], 0.0, 1e-5)
         assert_rel_error(self, data['comp'][('y', 'x')]['abs error'][1], 0.0, 1e-5)
@@ -540,7 +667,7 @@ class TestExecComp(unittest.TestCase):
         prob.set_solver_print(level=0)
         prob.run_model()
 
-        data = prob.check_partials()
+        data = prob.check_partials(out_stream=None)
 
         assert_rel_error(self, data['comp'][('y', 'x')]['abs error'][0], 0.0, 1e-5)
         assert_rel_error(self, data['comp'][('y', 'x')]['abs error'][1], 0.0, 1e-5)
@@ -660,7 +787,26 @@ class TestExecComp(unittest.TestCase):
         self.assertEqual(str(context.exception),
                          "comp: vectorize is True but partial(y, A) is not square (shape=(3, 15)).")
 
+    def test_vectorize_shape_only(self):
+        p = Problem()
+        model = p.model
+        model.add_subsystem('indep', IndepVarComp('x', val=np.ones(5)))
+
+        model.add_subsystem('comp', ExecComp('y=3.0*x + 2.5', vectorize=True,
+                                             x={'shape': (5,)}, y={'shape': (5,)}))
+        model.connect('indep.x', 'comp.x')
+
+        p.setup()
+        p.run_model()
+
+        J = p.compute_totals(of=['comp.y'], wrt=['indep.x'], return_format='array')
+
+        assert_almost_equal(J, np.eye(5)*3., decimal=6)
+
     def test_feature_vectorize(self):
+        import numpy as np
+        from openmdao.api import IndepVarComp, Problem, ExecComp
+
         p = Problem()
         model = p.model
         model.add_subsystem('indep', IndepVarComp('x', val=np.ones(5)))
@@ -796,6 +942,27 @@ class TestExecComp(unittest.TestCase):
 
         assert_rel_error(self, prob['comp.z'], 24.0, 0.00001)
 
+    def test_feature_options(self):
+        from openmdao.api import IndepVarComp, Group, Problem, ExecComp
+
+        model = Group()
+
+        indep = model.add_subsystem('indep', IndepVarComp('x', shape=(2,), units='cm'))
+        xcomp = model.add_subsystem('comp', ExecComp('y=2*x', shape=(2,)))
+
+        xcomp.options['units'] = 'm'
+
+        model.connect('indep.x', 'comp.x')
+
+        prob = Problem(model)
+        prob.setup()
+
+        prob['indep.x'] = [100., 200.]
+
+        prob.run_model()
+
+        assert_rel_error(self, prob['comp.y'], [2., 4.], 0.00001)
+
 
 class TestExecCompParameterized(unittest.TestCase):
 
@@ -840,7 +1007,7 @@ class TestExecCompParameterized(unittest.TestCase):
 
         if 'check_val' not in test_data:
             try:
-                prob.check_partials(compact_print=True)
+                prob.check_partials(out_stream=None)
             except TypeError as e:
                 print(f, 'does not support complex-step differentiation')
 
@@ -869,7 +1036,7 @@ class TestExecCompParameterized(unittest.TestCase):
         prob.run_model()
 
         if 'check_val' not in test_data:
-            cpd = prob.check_partials(compact_print=True)
+            cpd = prob.check_partials(out_stream=None)
 
             for comp in cpd:
                 for (var, wrt) in cpd[comp]:
