@@ -6,8 +6,9 @@ import numpy as np
 from six import itervalues, iteritems
 from six.moves import range
 
-from openmdao.core.component import Component
+from openmdao.core.component import Component, _full_slice
 from openmdao.utils.class_util import overrides_method
+from openmdao.utils.general_utils import ContainsAll
 from openmdao.recorders.recording_iteration_stack import Recording
 
 _inst_functs = ['compute_jacvec_product', 'compute_multi_jacvec_product']
@@ -57,6 +58,54 @@ class ExplicitComponent(Component):
             (new_jacvec_prod is not None and
              new_jacvec_prod != self._inst_functs['compute_jacvec_product']))
 
+    def _get_partials_varlists(self):
+        """
+        Get lists of 'of' and 'wrt' variables that form the partial jacobian.
+
+        Returns
+        -------
+        tuple(list, list)
+            'of' and 'wrt' variable lists.
+        """
+        of = list(self._var_allprocs_prom2abs_list['output'])
+        wrt = list(self._var_allprocs_prom2abs_list['input'])
+        return of, wrt
+
+    def _get_partials_var_sizes(self):
+        """
+        Get sizes of 'of' and 'wrt' variables that form the partial jacobian.
+
+        Returns
+        -------
+        tuple(ndarray, ndarray)
+            'of' and 'wrt' variable sizes.
+        """
+        iproc = self.comm.rank
+        out_sizes = self._var_sizes['nonlinear']['output'][iproc]
+        in_sizes = self._var_sizes['nonlinear']['input'][iproc]
+        return out_sizes, in_sizes
+
+    def _jacobian_wrt_iter(self, wrt_matches=None):
+        """
+        Iterate over (name, offset, end, idxs) for each column var in the systems's jacobian.
+
+        Parameters
+        ----------
+        wrt_matches : set or None
+            Only include row vars that are contained in this set.  This will determine what
+            the actual offsets are, i.e. the offsets will be into a reduced jacobian
+            containing only the matching columns.
+        """
+        if wrt_matches is None:
+            wrt_matches = ContainsAll()
+        abs2meta = self._var_allprocs_abs2meta
+        offset = end = 0
+        for wrt in self._var_allprocs_abs_names['input']:
+            if wrt in wrt_matches:
+                end += abs2meta[wrt]['size']
+                yield wrt, offset, end, _full_slice
+                offset = end
+
     def _setup_partials(self, recurse=True):
         """
         Call setup_partials in components.
@@ -84,9 +133,16 @@ class ExplicitComponent(Component):
                 if 'method' in self._subjacs_info[abs_key]:
                     del self._subjacs_info[abs_key]['method']
 
+            dct = {
+                'rows': arange,
+                'cols': arange,
+                'value': np.full(meta['size'], -1.),
+                'dependent': True,
+            }
+
             # ExplicitComponent jacobians have -1 on the diagonal.
-            self._declare_partials(out_name, out_name, rows=arange, cols=arange,
-                                   val=np.full(meta['size'], -1.))
+            if arange.size > 0:
+                self._declare_partials(out_name, out_name, dct)
 
     def _setup_jacobians(self, recurse=True):
         """
@@ -97,8 +153,8 @@ class ExplicitComponent(Component):
         recurse : bool
             If True, setup jacobians in all descendants. (ignored)
         """
-        if self._use_derivatives:
-            self._set_partials_meta()
+        if self._has_approx and self._use_derivatives:
+            self._set_approx_partials_meta()
 
     def add_output(self, name, val=1.0, shape=None, units=None, res_units=None, desc='',
                    lower=None, upper=None, ref=1.0, ref0=0.0, res_ref=None):
@@ -159,22 +215,13 @@ class ExplicitComponent(Component):
                                                          lower=lower, upper=upper,
                                                          ref=ref, ref0=ref0, res_ref=res_ref)
 
-    def _set_partials_meta(self):
-        """
-        Set subjacobian info into our jacobian.
-        """
+    def _approx_subjac_keys_iter(self):
         for abs_key, meta in iteritems(self._subjacs_info):
-
-            # if there isn't a declared partial value, set it to a dense matrix
-            if meta['value'] is None:
-                meta['value'] = np.zeros(meta['shape'])
-
             if 'method' in meta:
                 method = meta['method']
-                # Don't approximate output wrt output.``
                 if (method is not None and method in self._approx_schemes and abs_key[1]
                         not in self._outputs._views_flat):
-                    self._approx_schemes[method].add_approximation(abs_key, meta)
+                    yield abs_key
 
     def _apply_nonlinear(self):
         """
@@ -271,24 +318,25 @@ class ExplicitComponent(Component):
                         d_residuals.read_only = True
 
                     try:
+                        args = [self._inputs, d_inputs, d_residuals, mode]
+                        if self._discrete_inputs:
+                            args.append(self._discrete_inputs)
+
                         # We used to negate the residual here, and then re-negate after the hook
                         if d_inputs._ncol > 1:
                             if self.supports_multivecs:
-                                self.compute_multi_jacvec_product(self._inputs, d_inputs,
-                                                                  d_residuals, mode)
+                                self.compute_multi_jacvec_product(*args)
                             else:
                                 for i in range(d_inputs._ncol):
                                     # need to make the multivecs look like regular single vecs
                                     # since the component doesn't know about multivecs.
                                     d_inputs._icol = i
                                     d_residuals._icol = i
-                                    self.compute_jacvec_product(self._inputs, d_inputs,
-                                                                d_residuals, mode)
+                                    self.compute_jacvec_product(*args)
                                 d_inputs._icol = None
                                 d_residuals._icol = None
                         else:
-                            self.compute_jacvec_product(self._inputs, d_inputs,
-                                                        d_residuals, mode)
+                            self.compute_jacvec_product(*args)
                     finally:
                         self._inputs.read_only = False
                         d_inputs.read_only = d_residuals.read_only = False
@@ -345,9 +393,11 @@ class ExplicitComponent(Component):
         sub_do_ln : boolean
             Flag indicating if the children should call linearize on their linear solvers.
         """
-        if not self._has_compute_partials and not self._approx_schemes:
+        if not (self._has_compute_partials or self._approx_schemes):
             return
 
+        self._check_first_linearize()
+
         with self._unscaled_context(outputs=[self._outputs], residuals=[self._residuals]):
             # Computing the approximation before the call to compute_partials allows users to
             # override FD'd values.
@@ -363,7 +413,10 @@ class ExplicitComponent(Component):
                 # to absolute names (used by all jacobians internally).
                 try:
                     # We used to negate the jacobian here, and then re-negate after the hook.
-                    self.compute_partials(self._inputs, self._jacobian)
+                    if self._discrete_inputs:
+                        self.compute_partials(self._inputs, self._jacobian, self._discrete_inputs)
+                    else:
+                        self.compute_partials(self._inputs, self._jacobian)
                 finally:
                     self._inputs.read_only = False
 
@@ -384,7 +437,7 @@ class ExplicitComponent(Component):
         """
         pass
 
-    def compute_partials(self, inputs, partials):
+    def compute_partials(self, inputs, partials, discrete_inputs=None):
         """
         Compute sub-jacobian parts. The model is assumed to be in an unscaled state.
 
@@ -394,10 +447,12 @@ class ExplicitComponent(Component):
             unscaled, dimensional input variables read via inputs[key]
         partials : Jacobian
             sub-jac components written to partials[output_name, input_name]
+        discrete_inputs : dict or None
+            If not None, dict containing discrete input values.
         """
         pass
 
-    def compute_jacvec_product(self, inputs, d_inputs, d_outputs, mode):
+    def compute_jacvec_product(self, inputs, d_inputs, d_outputs, mode, discrete_inputs=None):
         r"""
         Compute jac-vector product. The model is assumed to be in an unscaled state.
 
@@ -416,5 +471,7 @@ class ExplicitComponent(Component):
             see outputs; product must be computed only if var_name in d_outputs
         mode : str
             either 'fwd' or 'rev'
+        discrete_inputs : dict or None
+            If not None, dict containing discrete input values.
         """
         pass
