@@ -3,7 +3,7 @@
 from __future__ import division
 
 import numpy as np
-from six import itervalues
+from six import itervalues, iteritems
 from six.moves import range
 
 from openmdao.core.component import Component
@@ -110,11 +110,28 @@ class ImplicitComponent(Component):
         Provide initial guess for states.
         """
         self._inputs.read_only = self._residuals.read_only = True
+        complex_step = self._inputs._under_complex_step
 
         try:
             with self._unscaled_context(outputs=[self._outputs], residuals=[self._residuals]):
-                self.guess_nonlinear(self._inputs, self._outputs, self._residuals)
+                if complex_step:
+                    self._inputs.set_complex_step_mode(False, keep_real=True)
+                    self._outputs.set_complex_step_mode(False, keep_real=True)
+                    self._residuals.set_complex_step_mode(False, keep_real=True)
+                if self._discrete_inputs or self._discrete_outputs:
+                    self.guess_nonlinear(self._inputs, self._outputs, self._residuals,
+                                         self._discrete_inputs, self._discrete_outputs)
+                else:
+                    self.guess_nonlinear(self._inputs, self._outputs, self._residuals)
         finally:
+            if complex_step:
+                # Note: passing in False swaps back to the complex vector, which is valid since
+                # the inputs and residuals value cannot be edited.
+                self._inputs.set_complex_step_mode(False)
+                self._inputs._under_complex_step = True
+                self._outputs.set_complex_step_mode(True)
+                self._residuals.set_complex_step_mode(False)
+                self._residuals._under_complex_step = True
             self._inputs.read_only = self._residuals.read_only = False
 
     def _apply_linear(self, jac, vec_names, rel_systems, mode, scope_out=None, scope_in=None):
@@ -241,6 +258,13 @@ class ImplicitComponent(Component):
                     finally:
                         d_outputs.read_only = d_residuals.read_only = False
 
+    def _approx_subjac_keys_iter(self):
+        for abs_key, meta in iteritems(self._subjacs_info):
+            if 'method' in meta:
+                method = meta['method']
+                if method is not None and method in self._approx_schemes:
+                    yield abs_key
+
     def _linearize(self, jac=None, sub_do_ln=True):
         """
         Compute jacobian / factorization. The model is assumed to be in a scaled state.
@@ -252,6 +276,8 @@ class ImplicitComponent(Component):
         sub_do_ln : boolean
             Flag indicating if the children should call linearize on their linear solvers.
         """
+        self._check_first_linearize()
+
         with self._unscaled_context(outputs=[self._outputs]):
             # Computing the approximation before the call to compute_partials allows users to
             # override FD'd values.
@@ -261,7 +287,11 @@ class ImplicitComponent(Component):
             self._inputs.read_only = self._outputs.read_only = True
 
             try:
-                self.linearize(self._inputs, self._outputs, self._jacobian)
+                if self._discrete_inputs or self._discrete_outputs:
+                    self.linearize(self._inputs, self._outputs, self._jacobian,
+                                   self._discrete_inputs, self._discrete_outputs)
+                else:
+                    self.linearize(self._inputs, self._outputs, self._jacobian)
             finally:
                 self._inputs.read_only = self._outputs.read_only = False
 
@@ -298,7 +328,8 @@ class ImplicitComponent(Component):
         """
         pass
 
-    def guess_nonlinear(self, inputs, outputs, residuals):
+    def guess_nonlinear(self, inputs, outputs, residuals,
+                        discrete_inputs=None, discrete_outputs=None):
         """
         Provide initial guess for states.
 
@@ -312,11 +343,14 @@ class ImplicitComponent(Component):
             unscaled, dimensional output variables read via outputs[key]
         residuals : Vector
             unscaled, dimensional residuals written to via residuals[key]
+        discrete_inputs : dict or None
+            If not None, dict containing discrete input values.
+        discrete_outputs : dict or None
+            If not None, dict containing discrete output values.
         """
         pass
 
-    def apply_linear(self, inputs, outputs,
-                     d_inputs, d_outputs, d_residuals, mode):
+    def apply_linear(self, inputs, outputs, d_inputs, d_outputs, d_residuals, mode):
         r"""
         Compute jac-vector product. The model is assumed to be in an unscaled state.
 
@@ -370,7 +404,7 @@ class ImplicitComponent(Component):
         else:  # rev
             d_residuals.set_vec(d_outputs)
 
-    def linearize(self, inputs, outputs, jacobian):
+    def linearize(self, inputs, outputs, jacobian, discrete_inputs=None, discrete_outputs=None):
         """
         Compute sub-jacobian parts and any applicable matrix factorizations.
 
@@ -384,6 +418,10 @@ class ImplicitComponent(Component):
             unscaled, dimensional output variables read via outputs[key]
         jacobian : Jacobian
             sub-jac components written to jacobian[output_name, input_name]
+        discrete_inputs : dict or None
+            If not None, dict containing discrete input values.
+        discrete_outputs : dict or None
+            If not None, dict containing discrete output values.
         """
         pass
 
