@@ -1,4 +1,4 @@
-""" Unit tests for the problem interface."""
+""" Unit tests for discrete variables."""
 
 import sys
 import unittest
@@ -15,11 +15,10 @@ from openmdao.api import Problem, IndepVarComp, NonlinearBlockGS, ScipyOptimizeD
     ImplicitComponent, ParallelGroup, BroydenSolver
 from openmdao.utils.assert_utils import assert_rel_error
 from openmdao.test_suite.components.paraboloid import Paraboloid
-from openmdao.test_suite.components.sellar import SellarDerivatives
-from openmdao.test_suite.components.sellar import SellarDerivativesGrouped, \
-     SellarNoDerivatives, SellarDerivatives, SellarStateConnection, StateConnection, \
+from openmdao.test_suite.components.sellar import StateConnection, \
      SellarDis1withDerivatives, SellarDis2withDerivatives
 
+
 class ModCompEx(ExplicitComponent):
     def __init__(self, modval, **kwargs):
         super(ModCompEx, self).__init__(**kwargs)
@@ -49,6 +48,40 @@ class ModCompIm(ImplicitComponent):
         discrete_outputs['y'] = discrete_inputs['x'] % self.modval
 
 
+class CompDiscWDerivs(ExplicitComponent):
+    def setup(self):
+        self.add_discrete_input('N', 2)
+        self.add_discrete_output('Nout', 2)
+        self.add_input('x')
+        self.add_output('y')
+        self.declare_partials('*', '*')
+
+    def compute(self, inputs, outputs, discrete_inputs, discrete_outputs):
+        discrete_outputs['Nout'] = discrete_inputs['N'] * 2
+        outputs['y'] = inputs['x'] * 3.
+
+    def compute_partials(self, inputs, partials, discrete_inputs):
+        partials['y', 'x'] = 3.
+
+
+class CompDiscWDerivsImplicit(StateConnection):
+    def setup(self):
+        super(CompDiscWDerivsImplicit, self).setup()
+        self.add_discrete_input('N', 2)
+        self.add_discrete_output('Nout', 2)
+
+    def apply_nonlinear(self, inputs, outputs, residuals, discrete_inputs, discrete_outputs):
+        super(CompDiscWDerivsImplicit, self).apply_nonlinear(inputs, outputs, residuals)
+        discrete_outputs['Nout'] = discrete_inputs['N'] * 2
+
+    def solve_nonlinear(self, inputs, outputs, discrete_inputs, discrete_outputs):
+        super(CompDiscWDerivsImplicit, self).solve_nonlinear(inputs, outputs)
+        discrete_outputs['Nout'] = discrete_inputs['N'] * 2
+
+    def linearize(self, inputs, outputs, J, discrets_inputs, discrete_outputs):
+        super(CompDiscWDerivsImplicit, self).linearize(inputs, outputs, J)
+
+
 class MixedCompDiscIn(ExplicitComponent):
     def __init__(self, mult, **kwargs):
         super(MixedCompDiscIn, self).__init__(**kwargs)
@@ -85,6 +118,16 @@ class InternalDiscreteGroup(Group):
         self.connect('C1.y', 'C2.x')
 
 
+class DiscreteDriver(Driver):
+
+    def __init__(self):
+        super(DiscreteDriver, self).__init__()
+        self.supports.declare('integer_design_vars', types=bool, default=True)
+
+    def run(self):
+        self.get_design_var_values()
+
+
 class _DiscreteVal(object):
     """Generic discrete value to test passing of objects."""
     def __init__(self, val):
@@ -224,6 +267,97 @@ class DiscreteTestCase(unittest.TestCase):
         self.assertEqual(str(ctx.exception),
                          "Type 'str' of output 'indep.x' is incompatible with type 'int' of input 'comp.x'.")
 
+    def test_driver_discrete_enforce_int(self):
+        # Drivers require discrete vars to be int or ndarrays of int.
+        prob = Problem()
+        model = prob.model
+
+        indep = model.add_subsystem('indep', IndepVarComp())
+        indep.add_discrete_output('x', 11)
+        model.add_subsystem('comp', ModCompIm(3))
+
+        model.connect('indep.x', 'comp.x')
+
+        model.add_design_var('indep.x', 11)
+        prob.driver = DiscreteDriver()
+        prob.setup()
+
+        # Insert a non integer
+        prob['indep.x'] = 3.7
+
+        with self.assertRaises(Exception) as ctx:
+            prob.run_driver()
+
+        msg = "Only integer scalars or ndarrays are supported as values for " + \
+              "discrete variables when used as a design variable. "
+        msg += "A value of type 'float' was specified."
+
+        self.assertEqual(str(ctx.exception), msg)
+
+        # Insert a float ndarray
+        prob['indep.x'] = np.array([3.0])
+
+        with self.assertRaises(Exception) as ctx:
+            prob.run_driver()
+
+        msg = "Only integer scalars or ndarrays are supported as values for " + \
+              "discrete variables when used as a design variable. "
+        msg += "An array of type 'float64' was specified."
+
+        self.assertEqual(str(ctx.exception), msg)
+
+        # Make sure these work.
+
+        prob['indep.x'] = np.array([3.0], dtype=np.int64)
+        prob.run_driver()
+
+        prob['indep.x'] = 5
+        prob.run_driver()
+
+    def test_discrete_deriv_explicit(self):
+        prob = Problem()
+        model = prob.model
+
+        indep = model.add_subsystem('indep', IndepVarComp())
+        indep.add_output('x', 1.0)
+
+        comp = model.add_subsystem('comp', CompDiscWDerivs())
+        model.connect('indep.x', 'comp.x')
+
+        model.add_design_var('indep.x')
+        model.add_objective('comp.y')
+
+        prob.setup()
+        prob.run_model()
+
+        J = prob.compute_totals(return_format='array')
+
+        np.testing.assert_almost_equal(J, np.array([[3.]]))
+
+    def test_discrete_deriv_implicit(self):
+        prob = Problem()
+        model = prob.model
+
+        indep = model.add_subsystem('indep', IndepVarComp())
+        indep.add_output('x', 1.0, ref=10.)
+        indep.add_discrete_output('N', 1)
+
+        comp = model.add_subsystem('comp', CompDiscWDerivsImplicit())
+        sink = model.add_subsystem('sink', MixedCompDiscIn(1.0))
+        model.connect('indep.x', 'comp.y2_actual')
+        model.connect('indep.N', 'comp.N')
+        model.connect('comp.Nout', 'sink.x')
+
+        model.add_design_var('indep.x')
+        model.add_objective('comp.y2_command')
+
+        prob.setup()
+        prob.run_model()
+
+        J = prob.compute_totals(return_format='array')
+
+        np.testing.assert_almost_equal(J, np.array([[-1]]))
+
     def test_deriv_err(self):
         prob = Problem()
         model = prob.model
@@ -261,6 +395,7 @@ class DiscreteTestCase(unittest.TestCase):
         self.assertEqual(str(ctx.exception),
                          "Total derivative with respect to 'indep.x' depends upon discrete output variables ['G.G1.C1.y'].")
 
+
 class SolverDiscreteTestCase(unittest.TestCase):
     def _setup_model(self, solver_class):
         prob = Problem()
@@ -391,9 +526,9 @@ class DiscretePromTestCase(unittest.TestCase):
         self.assertEqual(prob['C4.y'].getval(), 52)
 
 
-
 class DiscreteFeatureTestCase(unittest.TestCase):
     def test_feature_discrete(self):
+        import numpy as np
         from openmdao.api import Problem, IndepVarComp, ExplicitComponent
 
         class BladeSolidity(ExplicitComponent):
@@ -433,5 +568,6 @@ class DiscreteFeatureTestCase(unittest.TestCase):
         # minimum value
         assert_rel_error(self, prob['SolidityComp.blade_solidity'], 0.02984155, 1e-4)
 
+
 if __name__ == "__main__":
     unittest.main()
