@@ -25,31 +25,38 @@ else:
 
 class InOutArrayComp(ExplicitComponent):
 
-    def __init__(self, arr_size=10):
-        super(InOutArrayComp, self).__init__()
-        self.delay = 0.01
-        self.arr_size = arr_size
+    def initialize(self):
+        self.options.declare('arr_size', types=int, default=10,
+                             desc="Size of input and output vectors.")
+
+        self.options.declare('delay', types=float, default=.01,
+                             desc="Time to sleep in compute function.")
 
     def setup(self):
-        self.add_input('invec', np.ones(self.arr_size, float))
-        self.add_output('outvec', np.ones(self.arr_size, float))
+        arr_size = self.options['arr_size']
+
+        self.add_input('invec', np.ones(arr_size, float))
+        self.add_output('outvec', np.ones(arr_size, float))
 
     def compute(self, inputs, outputs):
-        time.sleep(self.delay)
+        time.sleep(self.options['delay'])
         outputs['outvec'] = inputs['invec'] * 2.
 
 
 class DistribCompSimple(ExplicitComponent):
     """Uses 2 procs but takes full input vars"""
 
-    def __init__(self, arr_size=10):
-        super(DistribCompSimple, self).__init__()
+    def initialize(self):
+        self.options['distributed'] = True
 
-        self.arr_size = arr_size
+        self.options.declare('arr_size', types=int, default=10,
+                             desc="Size of input and output vectors.")
 
     def setup(self):
-        self.add_input('invec', np.ones(self.arr_size, float))
-        self.add_output('outvec', np.ones(self.arr_size, float))
+        arr_size = self.options['arr_size']
+
+        self.add_input('invec', np.ones(arr_size, float))
+        self.add_output('outvec', np.ones(arr_size, float))
 
     def compute(self, inputs, outputs):
         if MPI and self.comm != MPI.COMM_NULL:
@@ -71,11 +78,12 @@ class DistribCompSimple(ExplicitComponent):
 class DistribInputComp(ExplicitComponent):
     """Uses 2 procs and takes input var slices"""
 
-    def __init__(self, arr_size=11):
-        super(DistribInputComp, self).__init__()
-        self.arr_size = arr_size
+    def initialize(self):
         self.options['distributed'] = True
 
+        self.options.declare('arr_size', types=int, default=11,
+                             desc="Size of input and output vectors.")
+
     def compute(self, inputs, outputs):
         if MPI:
             self.comm.Allgatherv(inputs['invec']*2.0,
@@ -88,23 +96,26 @@ class DistribInputComp(ExplicitComponent):
         comm = self.comm
         rank = comm.rank
 
-        self.sizes, self.offsets = evenly_distrib_idxs(comm.size, self.arr_size)
+        arr_size = self.options['arr_size']
+
+        self.sizes, self.offsets = evenly_distrib_idxs(comm.size, arr_size)
         start = self.offsets[rank]
         end = start + self.sizes[rank]
 
         self.add_input('invec', np.ones(self.sizes[rank], float),
                        src_indices=np.arange(start, end, dtype=int))
-        self.add_output('outvec', np.ones(self.arr_size, float), shape=np.int32(self.arr_size))
+        self.add_output('outvec', np.ones(arr_size, float), shape=np.int32(arr_size))
 
 
 class DistribOverlappingInputComp(ExplicitComponent):
     """Uses 2 procs and takes input var slices"""
 
-    def __init__(self, arr_size=11):
-        super(DistribOverlappingInputComp, self).__init__()
-        self.arr_size = arr_size
+    def initialize(self):
         self.options['distributed'] = True
 
+        self.options.declare('arr_size', types=int, default=11,
+                             desc="Size of input and output vectors.")
+
     def compute(self, inputs, outputs):
         outputs['outvec'][:] = 0
         if MPI:
@@ -123,6 +134,8 @@ class DistribOverlappingInputComp(ExplicitComponent):
         comm = self.comm
         rank = comm.rank
 
+        arr_size = self.options['arr_size']
+
         # need to initialize the input to have the correct local size
         if rank == 0:
             size = 8
@@ -133,7 +146,7 @@ class DistribOverlappingInputComp(ExplicitComponent):
             start = 4
             end = 11
 
-        self.add_output('outvec', np.zeros(self.arr_size, float))
+        self.add_output('outvec', np.zeros(arr_size, float))
         self.add_input('invec', np.ones(size, float),
                        src_indices=np.arange(start, end, dtype=int))
 
@@ -141,11 +154,12 @@ class DistribOverlappingInputComp(ExplicitComponent):
 class DistribInputDistribOutputComp(ExplicitComponent):
     """Uses 2 procs and takes input var slices."""
 
-    def __init__(self, arr_size=11):
-        super(DistribInputDistribOutputComp, self).__init__()
-        self.arr_size = arr_size
+    def initialize(self):
         self.options['distributed'] = True
 
+        self.options.declare('arr_size', types=int, default=11,
+                             desc="Size of input and output vectors.")
+
     def compute(self, inputs, outputs):
         outputs['outvec'] = inputs['invec']*2.0
 
@@ -154,7 +168,9 @@ class DistribInputDistribOutputComp(ExplicitComponent):
         comm = self.comm
         rank = comm.rank
 
-        sizes, offsets = evenly_distrib_idxs(comm.size, self.arr_size)
+        arr_size = self.options['arr_size']
+
+        sizes, offsets = evenly_distrib_idxs(comm.size, arr_size)
         start = offsets[rank]
         end = start + sizes[rank]
 
@@ -168,11 +184,12 @@ class DistribNoncontiguousComp(ExplicitComponent):
     var slices as well
     """
 
-    def __init__(self, arr_size=11):
-        super(DistribNoncontiguousComp, self).__init__()
-        self.arr_size = arr_size
+    def initialize(self):
         self.options['distributed'] = True
 
+        self.options.declare('arr_size', types=int, default=11,
+                             desc="Size of input and output vectors.")
+
     def compute(self, inputs, outputs):
         outputs['outvec'] = inputs['invec']*2.0
 
@@ -181,7 +198,9 @@ class DistribNoncontiguousComp(ExplicitComponent):
         comm = self.comm
         rank = comm.rank
 
-        idxs = list(take_nth(rank, comm.size, range(self.arr_size)))
+        arr_size = self.options['arr_size']
+
+        idxs = list(take_nth(rank, comm.size, range(arr_size)))
 
         self.add_input('invec', np.ones(len(idxs), float),
                        src_indices=idxs)
@@ -191,11 +210,12 @@ class DistribNoncontiguousComp(ExplicitComponent):
 class DistribGatherComp(ExplicitComponent):
     """Uses 2 procs gathers a distrib input into a full output"""
 
-    def __init__(self, arr_size=11):
-        super(DistribGatherComp, self).__init__()
-        self.arr_size = arr_size
+    def initialize(self):
         self.options['distributed'] = True
 
+        self.options.declare('arr_size', types=int, default=11,
+                             desc="Size of input and output vectors.")
+
     def compute(self, inputs, outputs):
         if MPI:
             self.comm.Allgatherv(inputs['invec'],
@@ -209,27 +229,31 @@ class DistribGatherComp(ExplicitComponent):
         comm = self.comm
         rank = comm.rank
 
+        arr_size = self.options['arr_size']
+
         self.sizes, self.offsets = evenly_distrib_idxs(comm.size,
-                                                       self.arr_size)
+                                                       arr_size)
         start = self.offsets[rank]
         end = start + self.sizes[rank]
 
         # need to initialize the variable to have the correct local size
         self.add_input('invec', np.ones(self.sizes[rank], float),
                        src_indices=np.arange(start, end, dtype=int))
-        self.add_output('outvec', np.ones(self.arr_size, float))
+        self.add_output('outvec', np.ones(arr_size, float))
 
 
 class NonDistribGatherComp(ExplicitComponent):
     """Uses 2 procs gathers a distrib output into a full input"""
 
-    def __init__(self, size):
-        super(NonDistribGatherComp, self).__init__()
-        self.size = size
+    def initialize(self):
+        self.options.declare('size', types=int, default=1,
+                             desc="Size of input and output vectors.")
 
     def setup(self):
-        self.add_input('invec', np.ones(self.size, float))
-        self.add_output('outvec', np.ones(self.size, float))
+        size = self.options['size']
+
+        self.add_input('invec', np.ones(size, float))
+        self.add_output('outvec', np.ones(size, float))
 
     def compute(self, inputs, outputs):
         outputs['outvec'] = inputs['invec']
@@ -243,8 +267,8 @@ class NOMPITests(unittest.TestCase):
 
         p = Problem(model=Group())
         top = p.model
-        C1 = top.add_subsystem("C1", InOutArrayComp(size))
-        C2 = top.add_subsystem("C2", DistribInputComp(size))
+        C1 = top.add_subsystem("C1", InOutArrayComp(arr_size=size))
+        C2 = top.add_subsystem("C2", DistribInputComp(arr_size=size))
         top.connect('C1.outvec', 'C2.invec')
 
         msg = "The 'distributed' option is set to True for Component C2, " \
@@ -274,8 +298,8 @@ class MPITests(unittest.TestCase):
 
         p = Problem(model=Group())
         top = p.model
-        C1 = top.add_subsystem("C1", InOutArrayComp(size))
-        C2 = top.add_subsystem("C2", DistribCompSimple(size))
+        C1 = top.add_subsystem("C1", InOutArrayComp(arr_size=size))
+        C2 = top.add_subsystem("C2", DistribCompSimple(arr_size=size))
         top.connect('C1.outvec', 'C2.invec')
 
         p.setup(check=False)
@@ -294,8 +318,8 @@ class MPITests(unittest.TestCase):
 
         p = Problem(model=Group())
         top = p.model
-        C1 = top.add_subsystem("C1", InOutArrayComp(size))
-        C2 = top.add_subsystem("C2", DistribInputComp(size))
+        C1 = top.add_subsystem("C1", InOutArrayComp(arr_size=size))
+        C2 = top.add_subsystem("C2", DistribInputComp(arr_size=size))
         top.connect('C1.outvec', 'C2.invec')
 
         p.setup(check=False)
@@ -314,8 +338,8 @@ class MPITests(unittest.TestCase):
 
         p = Problem(model=Group())
         top = p.model
-        C1 = top.add_subsystem("C1", InOutArrayComp(size))
-        C2 = top.add_subsystem("C2", DistribInputComp(size))
+        C1 = top.add_subsystem("C1", InOutArrayComp(arr_size=size))
+        C2 = top.add_subsystem("C2", DistribInputComp(arr_size=size))
         C3 = top.add_subsystem("C3", ExecComp("y=x", x=np.zeros(size*commsize),
                                               y=np.zeros(size*commsize)))
         top.connect('C1.outvec', 'C2.invec')
@@ -337,9 +361,9 @@ class MPITests(unittest.TestCase):
 
         p = Problem(model=Group())
         top = p.model
-        C1 = top.add_subsystem("C1", InOutArrayComp(size))
-        C2 = top.add_subsystem("C2", DistribInputDistribOutputComp(size))
-        C3 = top.add_subsystem("C3", DistribGatherComp(size))
+        C1 = top.add_subsystem("C1", InOutArrayComp(arr_size=size))
+        C2 = top.add_subsystem("C2", DistribInputDistribOutputComp(arr_size=size))
+        C3 = top.add_subsystem("C3", DistribGatherComp(arr_size=size))
         top.connect('C1.outvec', 'C2.invec')
         top.connect('C2.outvec', 'C3.invec')
         p.setup(check=False)
@@ -359,9 +383,9 @@ class MPITests(unittest.TestCase):
 
         p = Problem(model=Group())
         top = p.model
-        C1 = top.add_subsystem("C1", InOutArrayComp(size))
-        C2 = top.add_subsystem("C2", DistribNoncontiguousComp(size))
-        C3 = top.add_subsystem("C3", DistribGatherComp(size))
+        C1 = top.add_subsystem("C1", InOutArrayComp(arr_size=size))
+        C2 = top.add_subsystem("C2", DistribNoncontiguousComp(arr_size=size))
+        C3 = top.add_subsystem("C3", DistribGatherComp(arr_size=size))
         top.connect('C1.outvec', 'C2.invec')
         top.connect('C2.outvec', 'C3.invec')
         p.setup(check=False)
@@ -395,8 +419,8 @@ class MPITests(unittest.TestCase):
 
         p = Problem(model=Group())
         top = p.model
-        C1 = top.add_subsystem("C1", InOutArrayComp(size))
-        C2 = top.add_subsystem("C2", DistribOverlappingInputComp(size))
+        C1 = top.add_subsystem("C1", InOutArrayComp(arr_size=size))
+        C2 = top.add_subsystem("C2", DistribOverlappingInputComp(arr_size=size))
         top.connect('C1.outvec', 'C2.invec')
         p.setup(check=False)
 
@@ -421,9 +445,9 @@ class MPITests(unittest.TestCase):
 
         p = Problem(model=Group())
         top = p.model
-        C1 = top.add_subsystem("C1", InOutArrayComp(size))
-        C2 = top.add_subsystem("C2", DistribInputDistribOutputComp(size))
-        C3 = top.add_subsystem("C3", NonDistribGatherComp(size))
+        C1 = top.add_subsystem("C1", InOutArrayComp(arr_size=size))
+        C2 = top.add_subsystem("C2", DistribInputDistribOutputComp(arr_size=size))
+        C3 = top.add_subsystem("C3", NonDistribGatherComp(size=size))
         top.connect('C1.outvec', 'C2.invec')
         top.connect('C2.outvec', 'C3.invec')
         p.setup(check=False)
@@ -479,14 +503,14 @@ class DeprecatedMPITests(unittest.TestCase):
         p = Problem()
         top = p.model
 
-        C1 = top.add_subsystem("C1", InOutArrayComp(size))
+        C1 = top.add_subsystem("C1", InOutArrayComp(arr_size=size))
 
         # check deprecation on setter & getter
         msg = "The 'distributed' property provides backwards compatibility " \
               "with OpenMDAO <= 2.4.0 ; use the 'distributed' option instead."
 
         with assert_warning(DeprecationWarning, msg):
-            C2 = top.add_subsystem("C2", DeprecatedDistribInputComp(size))
+            C2 = top.add_subsystem("C2", DeprecatedDistribInputComp(arr_size=size))
 
         with assert_warning(DeprecationWarning, msg):
             C2.distributed
@@ -526,8 +550,8 @@ class ProbRemoteTests(unittest.TestCase):
         p = Problem(model=Group())
         top = p.model
         par = top.add_subsystem('par', ParallelGroup())
-        C1 = par.add_subsystem("C1", DistribInputDistribOutputComp(size))
-        C2 = par.add_subsystem("C2", DistribInputDistribOutputComp(size))
+        C1 = par.add_subsystem("C1", DistribInputDistribOutputComp(arr_size=size))
+        C2 = par.add_subsystem("C2", DistribInputDistribOutputComp(arr_size=size))
         p.setup(check=False)
 
         # Conclude setup but don't run model.
@@ -580,63 +604,75 @@ class MPIFeatureTests(unittest.TestCase):
         size = 15
 
         class DistribComp(ExplicitComponent):
-            def __init__(self, size):
-                super(DistribComp, self).__init__()
-                self.size = size
+            def initialize(self):
                 self.options['distributed'] = True
 
+                self.options.declare('size', types=int, default=1,
+                                     desc="Size of input and output vectors.")
+
+            def setup(self):
+                comm = self.comm
+                rank = comm.rank
+
+                size = self.options['size']
+
+                # results in 8 entries for proc 0 and 7 entries for proc 1 when using 2 processes.
+                sizes, offsets = evenly_distrib_idxs(comm.size, size)
+                start = offsets[rank]
+                end = start + sizes[rank]
+
+                self.add_input('invec', np.ones(sizes[rank], float),
+                               src_indices=np.arange(start, end, dtype=int))
+
+                self.add_output('outvec', np.ones(sizes[rank], float))
+
             def compute(self, inputs, outputs):
                 if self.comm.rank == 0:
                     outputs['outvec'] = inputs['invec'] * 2.0
                 else:
                     outputs['outvec'] = inputs['invec'] * -3.0
 
+        class Summer(ExplicitComponent):
+            """Sums a distributed input."""
+
+            def initialize(self):
+                self.options.declare('size', types=int, default=1,
+                                     desc="Size of input and output vectors.")
+
             def setup(self):
                 comm = self.comm
                 rank = comm.rank
 
-                # results in 8 entries for proc 0 and 7 entries for proc 1 when using 2 processes.
-                sizes, offsets = evenly_distrib_idxs(comm.size, self.size)
-                start = offsets[rank]
-                end = start + sizes[rank]
+                size = self.options['size']
 
-                self.add_input('invec', np.ones(sizes[rank], float),
-                               src_indices=np.arange(start, end, dtype=int))
-                self.add_output('outvec', np.ones(sizes[rank], float))
-
-        class Summer(ExplicitComponent):
-            """Sums a distributed input."""
-
-            def __init__(self, size):
-                super(Summer, self).__init__()
-                self.size = size
-
-            def setup(self):
                 # this results in 8 entries for proc 0 and 7 entries for proc 1
                 # when using 2 processes.
-                sizes, offsets = evenly_distrib_idxs(self.comm.size, self.size)
+                sizes, offsets = evenly_distrib_idxs(comm.size, size)
                 start = offsets[rank]
                 end = start + sizes[rank]
 
                 # NOTE: you must specify src_indices here for the input. Otherwise,
                 #       you'll connect the input to [0:local_input_size] of the
                 #       full distributed output!
-                self.add_input('invec', np.ones(sizes[self.comm.rank], float),
+                self.add_input('invec', np.ones(sizes[rank], float),
                                src_indices=np.arange(start, end, dtype=int))
+
                 self.add_output('out', 0.0)
 
             def compute(self, inputs, outputs):
                 data = np.zeros(1)
-                data[0] = np.sum(self._inputs['invec'])
+                data[0] = np.sum(inputs['invec'])
+
                 total = np.zeros(1)
                 self.comm.Allreduce(data, total, op=MPI.SUM)
-                self._outputs['out'] = total[0]
+
+                outputs['out'] = total[0]
 
         p = Problem(model=Group())
         top = p.model
         top.add_subsystem("indep", IndepVarComp('x', np.zeros(size)))
-        top.add_subsystem("C2", DistribComp(size))
-        top.add_subsystem("C3", Summer(size))
+        top.add_subsystem("C2", DistribComp(size=size))
+        top.add_subsystem("C3", Summer(size=size))
 
         top.connect('indep.x', 'C2.invec')
         top.connect('C2.outvec', 'C3.invec')
@@ -683,7 +719,6 @@ class TestGroupMPI(unittest.TestCase):
         p.model.add_subsystem('C1', MyComp(),
                               promotes_inputs=['x'])
 
-        p.set_solver_print(level=0)
         p.setup()
         p.run_model()
 
