@@ -2,7 +2,7 @@ from __future__ import print_function
 
 import unittest
 
-from six import assertRaisesRegex
+from six import assertRaisesRegex, iteritems
 from six.moves import range
 
 import itertools
@@ -15,7 +15,7 @@ except ImportError:
     from openmdao.utils.assert_utils import SkipParameterized as parameterized
 
 from openmdao.api import Problem, Group, IndepVarComp, ExecComp, ExplicitComponent, \
-    NonlinearRunOnce, NonLinearRunOnce
+    NonlinearRunOnce, NonLinearRunOnce, BalanceComp, NewtonSolver, DirectSolver
 from openmdao.utils.assert_utils import assert_rel_error, assert_warning
 from openmdao.test_suite.components.sellar import SellarDis2
 
@@ -143,6 +143,45 @@ class TestGroup(unittest.TestCase):
         self.assertEqual(p['a'], 3.0)
         self.assertEqual(p['comp1.b'], 6.0)
 
+    def test_inner_connect_w_extern_promote(self):
+        p = Problem()
+        g = p.model.add_subsystem('g', Group(), promotes_inputs=['c0.x'])
+        g.add_subsystem('ivc', IndepVarComp('x', 2.))
+        g.add_subsystem('c0', ExecComp('y = 2*x'))
+        g.connect('ivc.x', 'c0.x')
+
+        p.setup()
+        p.final_setup()
+
+        from openmdao.error_checking.check_config import _get_promoted_connected_ins
+        ins = _get_promoted_connected_ins(p.model)
+        self.assertEqual(len(ins), 1)
+        inp, tup = list(ins.items())[0]
+        in_proms, mans = tup
+        self.assertEqual(inp, 'g.c0.x')
+        self.assertEqual(in_proms, ['g'])
+        self.assertEqual(mans, [('c0.x', 'g')])
+
+    def test_inner_connect_w_2extern_promotes(self):
+        p = Problem()
+        g0 = p.model.add_subsystem('g0', Group(), promotes_inputs=['c0.x'])
+        g = g0.add_subsystem('g', Group(), promotes_inputs=['c0.x'])
+        g.add_subsystem('ivc', IndepVarComp('x', 2.))
+        g.add_subsystem('c0', ExecComp('y = 2*x'))
+        g.connect('ivc.x', 'c0.x')
+
+        p.setup()
+        p.final_setup()
+
+        from openmdao.error_checking.check_config import _get_promoted_connected_ins
+        ins = _get_promoted_connected_ins(p.model)
+        self.assertEqual(len(ins), 1)
+        inp, tup = list(ins.items())[0]
+        in_proms, mans = tup
+        self.assertEqual(inp, 'g0.g.c0.x')
+        self.assertEqual(list(sorted(in_proms)), ['g0', 'g0.g'])
+        self.assertEqual(mans, [('c0.x', 'g0.g')])
+
     def test_group_rename_connect(self):
         from openmdao.api import Problem, IndepVarComp, ExecComp
 
@@ -452,17 +491,24 @@ class TestGroup(unittest.TestCase):
         from openmdao.api import Problem, IndepVarComp, ExecComp
 
         p = Problem()
-        indep = p.model.add_subsystem('indep', IndepVarComp())
-        indep.add_output('x', np.ones(5), units='ft')
-        p.model.add_subsystem('C1', ExecComp('y=sum(x)', x={'value': np.zeros(5), 'units': 'inch'},
-                                             y={'units': 'inch'}))
-        p.model.connect('indep.x', 'C1.x')
-        p.set_solver_print(level=0)
+
+        indep_comp = IndepVarComp()
+        indep_comp.add_output('x', np.ones(5), units='ft')
+
+        exec_comp = ExecComp('y=sum(x)',
+                             x={'value': np.zeros(5), 'units': 'inch'},
+                             y={'units': 'inch'})
+
+        p.model.add_subsystem('indep', indep_comp)
+        p.model.add_subsystem('comp1', exec_comp)
+        p.model.connect('indep.x', 'comp1.x')
+
         p.setup()
         p.run_model()
+
         assert_rel_error(self, p['indep.x'], np.ones(5))
-        assert_rel_error(self, p['C1.x'], np.ones(5)*12.)
-        assert_rel_error(self, p['C1.y'], 60.)
+        assert_rel_error(self, p['comp1.x'], np.ones(5)*12.)
+        assert_rel_error(self, p['comp1.y'], 60.)
 
     def test_connect_1_to_many(self):
         import numpy as np
@@ -470,14 +516,17 @@ class TestGroup(unittest.TestCase):
         from openmdao.api import Problem, IndepVarComp, ExecComp
 
         p = Problem()
+
         p.model.add_subsystem('indep', IndepVarComp('x', np.ones(5)))
         p.model.add_subsystem('C1', ExecComp('y=sum(x)*2.0', x=np.zeros(5)))
         p.model.add_subsystem('C2', ExecComp('y=sum(x)*4.0', x=np.zeros(5)))
         p.model.add_subsystem('C3', ExecComp('y=sum(x)*6.0', x=np.zeros(5)))
+
         p.model.connect('indep.x', ['C1.x', 'C2.x', 'C3.x'])
-        p.set_solver_print(level=0)
+
         p.setup()
         p.run_model()
+
         assert_rel_error(self, p['C1.y'], 10.)
         assert_rel_error(self, p['C2.y'], 20.)
         assert_rel_error(self, p['C3.y'], 30.)
@@ -509,6 +558,7 @@ class TestGroup(unittest.TestCase):
         from openmdao.api import Problem, IndepVarComp, ExecComp
 
         p = Problem()
+
         p.model.add_subsystem('indep', IndepVarComp('x', np.ones(5)))
         p.model.add_subsystem('C1', ExecComp('y=sum(x)*2.0', x=np.zeros(3)))
         p.model.add_subsystem('C2', ExecComp('y=sum(x)*4.0', x=np.zeros(2)))
@@ -520,7 +570,6 @@ class TestGroup(unittest.TestCase):
         # use -2 (same as 3 in this case) to show that negative indices work.
         p.model.connect('indep.x', 'C2.x', src_indices=[-2, 4])
 
-        p.set_solver_print(level=0)
         p.setup()
         p.run_model()
 
@@ -535,6 +584,7 @@ class TestGroup(unittest.TestCase):
         from openmdao.api import Problem, IndepVarComp, ExecComp
 
         p = Problem()
+
         p.model.add_subsystem('indep', IndepVarComp('x', np.arange(12).reshape((4, 3))))
         p.model.add_subsystem('C1', ExecComp('y=sum(x)*2.0', x=np.zeros((2, 2))))
 
@@ -543,9 +593,9 @@ class TestGroup(unittest.TestCase):
                         src_indices=[[(0, 0), (-1, 1)],
                                      [(2, 1), (1, 1)]], flat_src_indices=False)
 
-        p.set_solver_print(level=0)
         p.setup()
         p.run_model()
+
         assert_rel_error(self, p['C1.x'], np.array([[0., 10.],
                                                     [7., 4.]]))
         assert_rel_error(self, p['C1.y'], 42.)
@@ -651,7 +701,6 @@ class TestGroup(unittest.TestCase):
         p.model.add_subsystem('C1', MyComp1(), promotes_inputs=['x'])
         p.model.add_subsystem('C2', MyComp2(), promotes_inputs=['x'])
 
-        p.set_solver_print(level=0)
         p.setup()
         p.run_model()
 
@@ -693,7 +742,6 @@ class TestGroup(unittest.TestCase):
         p.model.add_subsystem('C1', MyComp(),
                               promotes_inputs=['x'])
 
-        p.set_solver_print(level=0)
         p.setup()
         p.run_model()
 
@@ -795,7 +843,7 @@ class TestGroup(unittest.TestCase):
         assert_rel_error(self, p['C1.y'], 21.)
 
     def test_set_order_feature(self):
-        from openmdao.api import Problem, IndepVarComp
+        from openmdao.api import Problem, IndepVarComp, ExplicitComponent
 
         class ReportOrderComp(ExplicitComponent):
             """Adds name to list."""
@@ -818,9 +866,7 @@ class TestGroup(unittest.TestCase):
         model.add_subsystem('C2', ReportOrderComp(order_list))
         model.add_subsystem('C3', ReportOrderComp(order_list))
 
-        prob.set_solver_print(level=0)
-
-        prob.setup(check=False)
+        prob.setup()
         prob.run_model()
 
         self.assertEqual(order_list, ['C1', 'C2', 'C3'])
@@ -832,8 +878,9 @@ class TestGroup(unittest.TestCase):
         model.set_order(['indeps', 'C2', 'C1', 'C3'])
 
         # after changing the order, we must call setup again
-        prob.setup(check=False)
+        prob.setup()
         prob.run_model()
+
         self.assertEqual(order_list, ['C2', 'C1', 'C3'])
 
     def test_set_order(self):
@@ -908,6 +955,104 @@ class TestGroup(unittest.TestCase):
 
         # this test passes if it doesn't raise an exception
 
+    def test_guess_nonlinear_feature(self):
+        from openmdao.api import Problem, Group, ExecComp, IndepVarComp, BalanceComp, NewtonSolver, DirectSolver
+
+        class Discipline(Group):
+
+            def setup(self):
+                self.add_subsystem('comp0', ExecComp('y=x**2'))
+                self.add_subsystem('comp1', ExecComp('z=2*external_input'),
+                                   promotes_inputs=['external_input'])
+
+                self.add_subsystem('balance', BalanceComp('x', lhs_name='y', rhs_name='z'),
+                                   promotes_outputs=['x'])
+
+                self.connect('comp0.y', 'balance.y')
+                self.connect('comp1.z', 'balance.z')
+
+                self.connect('x', 'comp0.x')
+
+                self.nonlinear_solver = NewtonSolver(iprint=2, solve_subsystems=True)
+                self.linear_solver = DirectSolver()
+
+            def guess_nonlinear(self, inputs, outputs, residuals):
+                # inputs are addressed using full path name, regardless of promotion
+                external_input = inputs['comp1.external_input']
+
+                # balance drives x**2 = 2*external_input
+                x_guess = (2*external_input)**.5
+
+                # outputs are addressed by the their promoted names
+                outputs['x'] = x_guess # perfect guess should converge in 0 iterations
+
+        p = Problem()
+
+        p.model.add_subsystem('parameters', IndepVarComp('input_value', 1.))
+        p.model.add_subsystem('discipline', Discipline())
+
+        p.model.connect('parameters.input_value', 'discipline.external_input')
+
+        p.setup()
+        p.run_model()
+
+        self.assertEqual(p.model.nonlinear_solver._iter_count, 0)
+
+        assert_rel_error(self, p['discipline.x'], 1.41421356, 1e-6)
+
+    def test_guess_nonlinear_complex_step(self):
+
+        class Discipline(Group):
+
+            def setup(self):
+                self.add_subsystem('comp0', ExecComp('y=x**2'))
+                self.add_subsystem('comp1', ExecComp('z=2*external_input'),
+                                   promotes_inputs=['external_input'])
+
+                self.add_subsystem('balance', BalanceComp('x', lhs_name='y', rhs_name='z'),
+                                   promotes_outputs=['x'])
+
+                self.connect('comp0.y', 'balance.y')
+                self.connect('comp1.z', 'balance.z')
+
+                self.connect('x', 'comp0.x')
+
+                self.nonlinear_solver = NewtonSolver(iprint=2, solve_subsystems=True)
+                self.linear_solver = DirectSolver()
+
+            def guess_nonlinear(self, inputs, outputs, residuals):
+
+                if outputs._data.dtype == np.complex:
+                    raise RuntimeError('Vector should not be complex when guess_nonlinear is called.')
+
+                # inputs are addressed using full path name, regardless of promotion
+                external_input = inputs['comp1.external_input']
+
+                # balance drives x**2 = 2*external_input
+                x_guess = (2*external_input)**.5
+
+                # outputs are addressed by the their promoted names
+                outputs['x'] = x_guess # perfect guess should converge in 0 iterations
+
+        p = Problem()
+
+        p.model.add_subsystem('parameters', IndepVarComp('input_value', 1.))
+        p.model.add_subsystem('discipline', Discipline())
+
+        p.model.connect('parameters.input_value', 'discipline.external_input')
+
+        p.setup(force_alloc_complex=True)
+        p.run_model()
+
+        self.assertEqual(p.model.nonlinear_solver._iter_count, 0)
+
+        assert_rel_error(self, p['discipline.x'], 1.41421356, 1e-6)
+
+        totals = p.check_totals(of=['discipline.comp1.z'], wrt=['parameters.input_value'], method='cs', out_stream=None)
+
+        for key, val in iteritems(totals):
+            assert_rel_error(self, val['rel error'][0], 0.0, 1e-15)
+
 
 class MyComp(ExplicitComponent):
     def __init__(self, input_shape, src_indices=None, flat_src_indices=False):
