@@ -18,7 +18,6 @@ from openmdao.test_suite.components.sellar import SellarDerivatives, \
     SellarDis1withDerivatives, SellarDis2withDerivatives
 from openmdao.test_suite.groups.parallel_groups import FanOutGrouped, FanInGrouped
 from openmdao.utils.assert_utils import assert_rel_error
-from openmdao.recorders.recording_iteration_stack import recording_iteration
 
 
 if MPI:
@@ -205,7 +204,6 @@ class DecoupledTestCase(unittest.TestCase):
     def setup_model(self):
         asize = self.asize
         prob = Problem()
-        #import wingdbstub
         root = prob.model
         root.linear_solver = LinearBlockGS()
 
@@ -254,8 +252,6 @@ class DecoupledTestCase(unittest.TestCase):
         asize = self.asize
         prob = self.setup_model()
 
-        #import wingdbstub
-
         prob.model.add_design_var('Indep1.x', parallel_deriv_color='pardv')
         prob.model.add_design_var('Indep2.x', parallel_deriv_color='pardv')
         prob.model.add_constraint('Con1.y', upper=0.0)
@@ -276,8 +272,6 @@ class DecoupledTestCase(unittest.TestCase):
         asize = self.asize
         prob = self.setup_model()
 
-        #import wingdbstub
-
         prob.model.add_design_var('Indep1.x', parallel_deriv_color='pardv', vectorize_derivs=True)
         prob.model.add_design_var('Indep2.x', parallel_deriv_color='pardv', vectorize_derivs=True)
         prob.model.add_constraint('Con1.y', upper=0.0)
@@ -404,7 +398,7 @@ class IndicesTestCase2(unittest.TestCase):
 
         root.linear_solver = LinearBlockGS()
 
-        G1 = root.add_subsystem('G1', Group())# ParallelGroup())
+        G1 = root.add_subsystem('G1', ParallelGroup())
         G1.linear_solver = LinearBlockGS()
 
         par1 = G1.add_subsystem('par1', Group())
@@ -434,7 +428,7 @@ class IndicesTestCase2(unittest.TestCase):
         root.connect('G1.par1.c2.y', 'G1.par1.c4.x')
         root.connect('G1.par2.c3.y', 'G1.par2.c5.x')
 
-        prob.setup(check=False)
+        prob.setup(check=False, mode=mode)
         prob.run_driver()
 
         return prob
@@ -685,7 +679,6 @@ class ParDerivColorFeatureTestCase(unittest.TestCase):
         from openmdao.api import Problem
         from openmdao.core.tests.test_parallel_derivatives import PartialDependGroup
 
-        recording_iteration.stack = []
         size = 4
 
         of = ['ParallelGroup1.Con1.y', 'ParallelGroup1.Con2.y']
@@ -703,8 +696,6 @@ class ParDerivColorFeatureTestCase(unittest.TestCase):
         assert_rel_error(self, J['ParallelGroup1.Con1.y']['Indep1.x'][0], np.ones(size)*2., 1e-6)
         assert_rel_error(self, J['ParallelGroup1.Con2.y']['Indep1.x'][0], np.ones(size)*-3., 1e-6)
 
-        recording_iteration.stack = []
-
         # now run in rev mode and compare times for deriv calculation
         p = Problem(model=PartialDependGroup())
         p.setup(check=False, mode='rev')
