@@ -161,10 +161,11 @@ class ScipyOptimizeDriver(Driver):
                              'control, use solver-specific options.')
         self.options.declare('maxiter', 200, lower=0,
                              desc='Maximum number of iterations.')
-        self.options.declare('disp', True,
+        self.options.declare('disp', True, types=bool,
                              desc='Set to False to prevent printing of Scipy convergence messages')
         self.options.declare('dynamic_simul_derivs', default=False, types=bool,
-                             desc='Compute simultaneous derivative coloring dynamically if True')
+                             desc='Compute simultaneous derivative coloring dynamically if True '
+                             '(deprecated)')
         self.options.declare('dynamic_derivs_repeats', default=3, types=int,
                              desc='Number of compute_totals calls during dynamic computation of '
                                   'simultaneous derivative coloring')
@@ -178,7 +179,7 @@ class ScipyOptimizeDriver(Driver):
         str
             The name of the current optimizer.
         """
-        return self.options['optimizer']
+        return "ScipyOptimize_" + self.options['optimizer']
 
     def _setup_driver(self, problem):
         """
@@ -240,8 +241,8 @@ class ScipyOptimizeDriver(Driver):
         self._total_jac = None
 
         # Initial Run
-        with RecordingDebugging(self.options['optimizer'], self.iter_count, self) as rec:
-            model._solve_nonlinear()
+        with RecordingDebugging(self._get_name(), self.iter_count, self) as rec:
+            model.run_solve_nonlinear()
             self.iter_count += 1
 
         self._con_cache = self.get_constraint_values()
@@ -412,8 +413,15 @@ class ScipyOptimizeDriver(Driver):
             hess = None
 
         # compute dynamic simul deriv coloring if option is set
-        if coloring_mod._use_sparsity and self.options['dynamic_simul_derivs']:
-            coloring_mod.dynamic_simul_coloring(self, run_model=False, do_sparsity=False)
+        if coloring_mod._use_sparsity:
+            if self._coloring_info['coloring'] is coloring_mod._DYN_COLORING:
+                coloring_mod.dynamic_total_coloring(self, run_model=False,
+                                                    fname=self._get_total_coloring_fname())
+            elif self.options['dynamic_simul_derivs']:
+                warn_deprecation("The 'dynamic_simul_derivs' option has been deprecated. Call "
+                                 "the 'declare_coloring' function instead.")
+                coloring_mod.dynamic_total_coloring(self, run_model=False,
+                                                    fname=self._get_total_coloring_fname())
 
         # optimize
         try:
@@ -555,9 +563,9 @@ class ScipyOptimizeDriver(Driver):
                 self.set_design_var(name, x_new[i:i + size])
                 i += size
 
-            with RecordingDebugging(self.options['optimizer'], self.iter_count, self) as rec:
+            with RecordingDebugging(self._get_name(), self.iter_count, self) as rec:
                 self.iter_count += 1
-                model._solve_nonlinear()
+                model.run_solve_nonlinear()
 
             # Get the objective function evaluations
             for obj in itervalues(self.get_objective_values()):
