@@ -1659,6 +1659,9 @@ class TestPyoptSparse(unittest.TestCase):
 class TestPyoptSparseFeature(unittest.TestCase):
 
     def setUp(self):
+        from openmdao.utils.general_utils import set_pyoptsparse_opt
+        import unittest
+
         OPT, OPTIMIZER = set_pyoptsparse_opt('SLSQP')
         if OPT is None:
             raise unittest.SkipTest("pyoptsparse is not installed")
@@ -1773,6 +1776,8 @@ class TestPyoptSparseSnoptFeature(unittest.TestCase):
     # all of these tests require SNOPT
 
     def setUp(self):
+        from openmdao.utils.general_utils import set_pyoptsparse_opt
+
         OPT, OPTIMIZER = set_pyoptsparse_opt('SNOPT', fallback=False)
 
     def test_snopt_atol(self):
@@ -1814,7 +1819,8 @@ class TestPyoptSparseSnoptFeature(unittest.TestCase):
         prob.driver = pyOptSparseDriver()
         prob.driver.options['optimizer'] = "SNOPT"
 
-        prob.driver.opt_settings['Major iterations limit'] = 4
+        # after upgrading to SNOPT 7.5-1.1, this test failed unless iter limit raised from 4 to 5
+        prob.driver.opt_settings['Major iterations limit'] = 5
 
         model.add_design_var('z', lower=np.array([-10.0, 0.0]), upper=np.array([10.0, 10.0]))
         model.add_design_var('x', lower=0.0, upper=10.0)
@@ -1828,7 +1834,7 @@ class TestPyoptSparseSnoptFeature(unittest.TestCase):
 
         prob.run_driver()
 
-        assert_rel_error(self, prob['z'][0], 1.9780247, 1e-3)
+        assert_rel_error(self, prob['z'][0], 1.9780247, 2e-3)
 
     def test_snopt_fd_solution(self):
 
