@@ -115,6 +115,32 @@ class Matrix(object):
         """
         pass
 
+    def _pre_update(self):
+        """
+        Do anything that needs to be done at the beginning of AssembledJacobian._update.
+        """
+        pass
+
+    def _post_update(self):
+        """
+        Do anything that needs to be done at the end of AssembledJacobian._update.
+        """
+        pass
+
+    def set_complex_step_mode(self, active):
+        """
+        Turn on or off complex stepping mode.
+
+        When turned on, the value in each subjac is cast as complex, and when turned
+        off, they are returned to real values.
+
+        Parameters
+        ----------
+        active : bool
+            Complex mode flag; set to True prior to commencing complex step.
+        """
+        pass
+
 
 def _compute_index_map(jrows, jcols, irow, icol, src_indices):
     """
