@@ -185,7 +185,7 @@ class TestSqliteRecorder(unittest.TestCase):
         t0, t1 = run_driver(prob)
         prob.cleanup()
 
-        coordinate = [0, 'SLSQP', (4, )]
+        coordinate = [0, 'ScipyOptimize_SLSQP', (4, )]
 
         expected_desvars = {"p1.x": [7.16706813], "p2.y": [-7.83293187]}
         expected_objectives = {"comp.f_xy": [-27.0833]}
@@ -261,7 +261,7 @@ class TestSqliteRecorder(unittest.TestCase):
         t0, t1 = run_driver(prob)
         prob.cleanup()
 
-        coordinate = [0, 'SLSQP', (3, )]
+        coordinate = [0, 'pyOptSparse_SLSQP', (3, )]
 
         expected_desvars = {"p1.x": [7.16706813], "p2.y": [-7.83293187]}
         expected_objectives = {"comp.f_xy": [-27.0833]}
@@ -309,8 +309,8 @@ class TestSqliteRecorder(unittest.TestCase):
         run2_t0, run2_t1 = run_driver(prob, case_prefix='Run2')
         prob.cleanup()
 
-        run1_coord = [0, 'SLSQP', (4, )]  # 1st run, 5 iterations
-        run2_coord = [0, 'SLSQP', (0, )]  # 2nd run, 1 iteration
+        run1_coord = [0, 'ScipyOptimize_SLSQP', (4, )]  # 1st run, 5 iterations
+        run2_coord = [0, 'ScipyOptimize_SLSQP', (0, )]  # 2nd run, 1 iteration
 
         expected_desvars = {"p1.x": [7.16706813], "p2.y": [-7.83293187]}
         expected_objectives = {"comp.f_xy": [-27.0833]}
@@ -361,7 +361,7 @@ class TestSqliteRecorder(unittest.TestCase):
         t0, t1 = run_driver(prob)
         prob.cleanup()
 
-        coordinate = [0, 'SLSQP', (3, )]
+        coordinate = [0, 'ScipyOptimize_SLSQP', (3, )]
 
         expected_desvars = {"p1.x": [7.16706813, ], "p2.y": [-7.83293187]}
         expected_objectives = {"comp.f_xy": [-27.0833]}
@@ -440,7 +440,7 @@ class TestSqliteRecorder(unittest.TestCase):
         assertMetadataRecorded(self, prom2abs, abs2prom)
         expected_problem_metadata = {
             'connections_list_length': 11,
-            'tree_length': 6,
+            'tree_length': 8,
             'tree_children_length': 7,
             'abs2prom': abs2prom,
         }
@@ -643,7 +643,7 @@ class TestSqliteRecorder(unittest.TestCase):
 
         prob.cleanup()
 
-        coordinate = [0, 'SLSQP', (3, )]
+        coordinate = [0, 'ScipyOptimize_SLSQP', (3, )]
 
         expected_desvars = {"p1.x": prob["p1.x"]}
         expected_objectives = {"comp.f_xy": prob['comp.f_xy']}
@@ -686,7 +686,7 @@ class TestSqliteRecorder(unittest.TestCase):
         t0, t1 = run_driver(prob)
         prob.cleanup()
 
-        coordinate = [0, 'SLSQP', (3, )]
+        coordinate = [0, 'ScipyOptimize_SLSQP', (3, )]
 
         expected_desvars = {"p1.x": prob["p1.x"]}
         expected_objectives = {"comp.f_xy": prob['comp.f_xy']}
@@ -743,7 +743,7 @@ class TestSqliteRecorder(unittest.TestCase):
         #
         coordinate = [
             0,
-            'SLSQP', (1, ),
+            'ScipyOptimize_SLSQP', (1, ),
             'root._solve_nonlinear', (1, ),
             'NLRunOnce', (0, ),
             'mda._solve_nonlinear', (1, ),
@@ -767,7 +767,7 @@ class TestSqliteRecorder(unittest.TestCase):
         #
         # check data for 'pz'
         #
-        coordinate = [0, 'SLSQP', (2, ), 'root._solve_nonlinear', (2, ), 'NLRunOnce', (0, ),
+        coordinate = [0, 'ScipyOptimize_SLSQP', (2, ), 'root._solve_nonlinear', (2, ), 'NLRunOnce', (0, ),
                       'pz._solve_nonlinear', (2, )]
 
         expected_inputs = None
@@ -1164,7 +1164,7 @@ class TestSqliteRecorder(unittest.TestCase):
         #
         # Driver recording test
         #
-        coordinate = [0, 'SLSQP', (6, )]
+        coordinate = [0, 'ScipyOptimize_SLSQP', (6, )]
 
         expected_desvars = {
             "pz.z": prob['pz.z'],
@@ -1188,7 +1188,7 @@ class TestSqliteRecorder(unittest.TestCase):
         #
         # System recording test
         #
-        coordinate = [0, 'SLSQP', (2, ), 'root._solve_nonlinear', (2, ), 'NLRunOnce', (0, ),
+        coordinate = [0, 'ScipyOptimize_SLSQP', (2, ), 'root._solve_nonlinear', (2, ), 'NLRunOnce', (0, ),
                       'pz._solve_nonlinear', (2, )]
 
         expected_inputs = None
@@ -1203,7 +1203,7 @@ class TestSqliteRecorder(unittest.TestCase):
         #
         # Solver recording test
         #
-        coordinate = [0, 'SLSQP', (6, ), 'root._solve_nonlinear', (6, ), 'NLRunOnce', (0, ),
+        coordinate = [0, 'ScipyOptimize_SLSQP', (6, ), 'root._solve_nonlinear', (6, ), 'NLRunOnce', (0, ),
                       'mda._solve_nonlinear', (6, ), 'NonlinearBlockGS', (4, )]
 
         expected_abs_error = 0.0,
@@ -1460,7 +1460,7 @@ class TestSqliteRecorder(unittest.TestCase):
         prob.cleanup()
 
         # Driver recording test
-        coordinate = [0, 'SLSQP', (6, )]
+        coordinate = [0, 'ScipyOptimize_SLSQP', (6, )]
 
         expected_desvars = {
             "pz.z": prob['pz.z'],
@@ -1577,6 +1577,37 @@ class TestSqliteRecorder(unittest.TestCase):
         self.assertFalse(system._rec_mgr.has_recorders())
         self.assertFalse(solver._rec_mgr.has_recorders())
 
+    def test_problem_record_no_voi(self):
+        prob = Problem(model=SellarDerivatives())
+
+        prob.add_recorder(SqliteRecorder("cases.sql"))
+
+        prob.setup()
+        prob.run_driver()
+
+        prob.record_iteration('final')
+        prob.cleanup()
+
+        cr = CaseReader("cases.sql")
+
+        problem_cases = cr.list_cases('problem')
+        self.assertEqual(len(problem_cases), 1)
+
+        final_case = cr.get_case('final')
+
+        # we didn't declare any VOIs
+        desvars = final_case.get_design_vars()
+        objectives = final_case.get_objectives()
+        constraints = final_case.get_constraints()
+
+        self.assertEqual(len(desvars), 0)
+        self.assertEqual(len(objectives), 0)
+        self.assertEqual(len(constraints), 0)
+
+        # by default we should get all outputs
+        self.assertEqual(set(final_case.outputs.keys()),
+                         {'con1', 'con2', 'obj', 'x', 'y1', 'y2', 'z'})
+
     def test_problem_record_with_options(self):
         prob = Problem(model=SellarDerivatives())
 
@@ -1590,7 +1621,6 @@ class TestSqliteRecorder(unittest.TestCase):
 
         prob.add_recorder(SqliteRecorder("cases.sql"))
 
-        prob.recording_options['includes'] = []
         prob.recording_options['record_objectives'] = False
         prob.recording_options['record_constraints'] = False
         prob.recording_options['record_desvars'] = False
@@ -1616,6 +1646,49 @@ class TestSqliteRecorder(unittest.TestCase):
         self.assertEqual(len(objectives), 0)
         self.assertEqual(len(constraints), 0)
 
+        # includes all outputs (default) minus the VOIs, which we have excluded
+        self.assertEqual(set(final_case.outputs.keys()), {'y1', 'y2'})
+
+    def test_problem_record_options_includes(self):
+        prob = Problem(model=SellarDerivatives())
+
+        model = prob.model
+        model.add_design_var('z', lower=np.array([-10.0, 0.0]),
+                                  upper=np.array([10.0, 10.0]))
+        model.add_design_var('x', lower=0.0, upper=10.0)
+        model.add_objective('obj')
+        model.add_constraint('con1', upper=0.0)
+        model.add_constraint('con2', upper=0.0)
+
+        prob.add_recorder(SqliteRecorder("cases.sql"))
+
+        prob.recording_options['includes'] = []
+
+        prob.setup()
+        prob.run_driver()
+
+        prob.record_iteration('final')
+        prob.cleanup()
+
+        cr = CaseReader("cases.sql")
+
+        problem_cases = cr.list_cases('problem')
+        self.assertEqual(len(problem_cases), 1)
+
+        final_case = cr.get_case('final')
+
+        desvars = final_case.get_design_vars()
+        objectives = final_case.get_objectives()
+        constraints = final_case.get_constraints()
+
+        self.assertEqual(len(desvars), 2)
+        self.assertEqual(len(objectives), 1)
+        self.assertEqual(len(constraints), 2)
+
+        # includes no outputs except the the VOIs that are recorded by default
+        self.assertEqual(set(final_case.outputs.keys()),
+                         {'con1', 'con2', 'obj', 'x', 'z'})
+
     def test_simple_paraboloid_scaled_desvars(self):
         prob = Problem()
         model = prob.model
@@ -1670,11 +1743,16 @@ class TestSqliteRecorder(unittest.TestCase):
 
 class TestFeatureSqliteRecorder(unittest.TestCase):
     def setUp(self):
+        import os
+        from tempfile import mkdtemp
         self.dir = mkdtemp()
         self.original_path = os.getcwd()
         os.chdir(self.dir)
 
     def tearDown(self):
+        import os
+        import errno
+        from shutil import rmtree
         os.chdir(self.original_path)
         try:
             rmtree(self.dir)
@@ -1720,7 +1798,7 @@ class TestFeatureSqliteRecorder(unittest.TestCase):
         prob.cleanup()
 
         cr = CaseReader(case_recorder_filename)
-        case = cr.get_case('rank0:SLSQP|4')
+        case = cr.get_case('rank0:ScipyOptimize_SLSQP|4')
 
         assert_rel_error(self, case.outputs['x'], 7.16666667, 1e-6)
         assert_rel_error(self, case.outputs['y'], -7.83333333, 1e-6)
@@ -1757,13 +1835,65 @@ class TestFeatureSqliteRecorder(unittest.TestCase):
 
         # access the model tree stored in metadata
         self.assertEqual(set(cr.problem_metadata['tree'].keys()),
-                         {'name', 'type', 'subsystem_type', 'children', 'linear_solver', 'nonlinear_solver'})
+                         {'name', 'type', 'subsystem_type', 'children', 'linear_solver',
+                          'nonlinear_solver', 'is_parallel', 'component_type'})
         self.assertEqual(cr.problem_metadata['tree']['name'], 'root')
         self.assertEqual(sorted([child["name"] for child in cr.problem_metadata['tree']["children"]]),
                          ['con_cmp1', 'con_cmp2', 'd1', 'd2', 'obj_cmp', 'px', 'pz'])
 
+    def test_feature_problem_metadata_with_driver_information(self):
+        from openmdao.api import Problem, ScipyOptimizeDriver, SqliteRecorder, CaseReader
+        from openmdao.api import DOEDriver, UniformGenerator 
+        from openmdao.test_suite.components.sellar import SellarDerivatives
+
+        prob = Problem(SellarDerivatives())
+        model = prob.model
+        model.add_design_var('z', lower=np.array([-10.0, 0.0]),
+                                  upper=np.array([10.0, 10.0]))
+        model.add_design_var('x', lower=0.0, upper=10.0)
+        model.add_objective('obj')
+        model.add_constraint('con1', upper=0.0)
+        model.add_constraint('con2', upper=0.0)
+
+        # DOE
+        driver = prob.driver = DOEDriver(UniformGenerator())
+        recorder = SqliteRecorder("cases.sql")
+        prob.driver.add_recorder(recorder)
+        prob.setup()
+        prob.run_driver()
+        prob.cleanup()
+
+        cr = CaseReader("cases.sql")
+        metadata = cr.problem_metadata['driver']
+        self.assertEqual(set(metadata.keys()), {'name', 'type', 'options', 'opt_settings'})
+        self.assertEqual(metadata['name'], 'DOEDriver')
+        self.assertEqual(metadata['type'], 'doe')
+        self.assertEqual(metadata['options'], {'debug_print': [], 'generator': 'UniformGenerator', 
+                                               'run_parallel': False, 'procs_per_model': 1}) 
+
+        # Optimization
+        driver = prob.driver = ScipyOptimizeDriver()
+        recorder = SqliteRecorder("cases.sql")
+        driver.options['optimizer'] = 'SLSQP'
+        driver.options['tol'] = 1e-3
+        driver.opt_settings['ACC'] = 1e-6
+        prob.driver.add_recorder(recorder)
+        prob.setup()
+        prob.run_driver()
+        prob.cleanup()
+
+        cr = CaseReader("cases.sql")
+        metadata = cr.problem_metadata['driver']
+        self.assertEqual(set(metadata.keys()), {'name', 'type', 'options', 'opt_settings'})
+        self.assertEqual(metadata['name'], 'ScipyOptimizeDriver')
+        self.assertEqual(metadata['type'], 'optimization')
+        self.assertEqual(metadata['options'], {"debug_print": [], "optimizer": "SLSQP", 
+                                               "tol": 1e-03, "maxiter": 200, "disp": True, 
+                                               "dynamic_simul_derivs": False, "dynamic_derivs_repeats": 3}) 
+        self.assertEqual(metadata['opt_settings'], {"ACC": 1e-06})
+
     def test_feature_solver_metadata(self):
-        from openmdao.api import Problem, SqliteRecorder, CaseReader
+        from openmdao.api import Problem, SqliteRecorder, CaseReader, NonlinearBlockGS
         from openmdao.test_suite.components.sellar import SellarDerivatives
 
         prob = Problem(model=SellarDerivatives())
@@ -1809,10 +1939,9 @@ class TestFeatureSqliteRecorder(unittest.TestCase):
         # declare two options
         d1 = prob.model.d1
         d1.options.declare('options value 1', 1)
-        d1.options.declare('options value to ignore', 2)
 
-        # don't record the second option on d1
-        d1.recording_options['options_excludes'] = ['options value to ignore']
+        # don't record the dynamic_derivs_repeats option on d1
+        d1.recording_options['options_excludes'] = ['dynamic_derivs_repeats']
 
         # create recorder and attach to driver and d1
         recorder = SqliteRecorder("cases.sql")
@@ -1831,17 +1960,8 @@ class TestFeatureSqliteRecorder(unittest.TestCase):
                          sorted(['root', 'px', 'pz', 'd1', 'd2', 'obj_cmp', 'con_cmp1', 'con_cmp2']))
 
         # options for system 'd1', with second option excluded
-        self.assertEqual(str(metadata['d1']['component_options']),
-            "=============== ======= ================= ================ =========================================\n"
-            "Option          Default Acceptable Values Acceptable Types Description                              \n"
-            "=============== ======= ================= ================ =========================================\n"
-            "distributed     False   N/A               N/A              True if the component has variables that \n"
-            "                                                           are distributed across multiple processes\n"
-            "                                                           .\n"
-            "options value 1 1       N/A               N/A                                                       \n"
-            "=============== ======= ================= ================ =========================================")
-
         self.assertEqual(metadata['d1']['component_options']['distributed'], False)
+        self.assertEqual(metadata['d1']['component_options']['options value 1'], 1)
 
     def test_feature_system_options(self):
         from openmdao.api import Problem, SqliteRecorder, CaseReader
@@ -1947,7 +2067,7 @@ class TestFeatureSqliteRecorder(unittest.TestCase):
     def test_feature_circuit_with_recorder(self):
         from openmdao.api import Group, NewtonSolver, DirectSolver, Problem, IndepVarComp, \
             CaseReader, SqliteRecorder
-        from openmdao.test_suite.test_examples.test_circuit_analysis import Resistor, Diode, Node
+        from openmdao.test_suite.scripts.circuit_analysis import Resistor, Diode, Node
 
         class Circuit(Group):
 
@@ -2173,9 +2293,66 @@ class TestFeatureSqliteRecorder(unittest.TestCase):
         assert_rel_error(self, design_vars, case.get_design_vars(), 1e-1)
         assert_rel_error(self, constraints, case.get_constraints(), 1e-1)
 
+    def test_scaling_multiple_calls(self):
+        from openmdao.api import Problem, SqliteRecorder, ScipyOptimizeDriver, CaseReader
+        from openmdao.test_suite.components.sellar import SellarDerivatives
+
+        import numpy as np
+
+        scaler = 2.
+
+        prob = Problem(model=SellarDerivatives())
+
+        model = prob.model
+        model.add_design_var('z', lower=np.array([-10.0, 0.0]),
+                                  upper=np.array([10.0, 10.0]))
+        model.add_design_var('x', lower=0.0, upper=10.0)
+        model.add_objective('obj', scaler=scaler)
+        model.add_constraint('con1', upper=0.0, scaler=scaler)
+        model.add_constraint('con2', upper=0.0, scaler=scaler)
+
+        prob.driver = ScipyOptimizeDriver(optimizer='SLSQP', tol=1e-9)
+
+        prob.add_recorder(SqliteRecorder("cases.sql"))
+
+        prob.recording_options['includes'] = []
+        prob.recording_options['record_objectives'] = True
+        prob.recording_options['record_constraints'] = True
+        prob.recording_options['record_desvars'] = True
+
+        prob.setup()
+        prob.run_driver()
+        prob.record_iteration('final')
+        prob.cleanup()
+
+        cr = CaseReader("cases.sql")
+
+        # get list of cases recorded on problem
+        problem_cases = cr.list_cases('problem')
+        self.assertEqual(problem_cases, ['final'])
+
+        # get list of output variables recorded on problem
+        problem_vars = cr.list_source_vars('problem')
+        self.assertEqual(sorted(problem_vars['outputs']), ['con1', 'con2', 'obj', 'x', 'z'])
+
+        # get the recorded case and check values
+        case = cr.get_case('final')
+
+        objectives = case.get_objectives()
+        design_vars = case.get_design_vars()
+        constraints = case.get_constraints()
+
+        # Methods are called a second time
+        assert_rel_error(self, objectives['obj'], case.get_objectives()['obj'], 1e-1)
+        assert_rel_error(self, design_vars, case.get_design_vars(), 1e-1)
+        assert_rel_error(self, constraints, case.get_constraints(), 1e-1)
+
 
 class TestFeatureBasicRecording(unittest.TestCase):
     def setUp(self):
+        import os
+        from tempfile import mkdtemp
+
         self.dir = mkdtemp()
         self.original_path = os.getcwd()
         os.chdir(self.dir)
@@ -2183,6 +2360,10 @@ class TestFeatureBasicRecording(unittest.TestCase):
         self.record_cases()
 
     def tearDown(self):
+        import os
+        import errno
+        from shutil import rmtree
+
         os.chdir(self.original_path)
         try:
             rmtree(self.dir)
