@@ -78,7 +78,7 @@ class BoundsEnforceLS(NonlinearSolver):
         super(BoundsEnforceLS, self)._declare_options()
         opt = self.options
         opt.declare(
-            'bound_enforcement', default='vector', values=['vector', 'scalar', 'wall'],
+            'bound_enforcement', default='scalar', values=['vector', 'scalar', 'wall'],
             desc="If this is set to 'vector', then the output vector is backtracked to the "
             "first point where violation occured. If it is set to 'scalar' or 'wall', then only "
             "the violated variables are backtracked to their point of violation.")
@@ -93,18 +93,9 @@ class BoundsEnforceLS(NonlinearSolver):
         opt.undeclare("maxiter")
         opt.undeclare("err_on_maxiter")
 
-    def _run_iterator(self):
+    def _solve(self):
         """
         Run the iterative solver.
-
-        Returns
-        -------
-        boolean
-            Failure flag; True if failed to converge, False is successful.
-        float
-            absolute error.
-        float
-            relative error.
         """
         self._iter_count = 0
         system = self._system
@@ -142,9 +133,6 @@ class BoundsEnforceLS(NonlinearSolver):
 
         self._mpi_print(self._iter_count, norm, norm / norm0)
 
-        fail = (np.isinf(norm) or np.isnan(norm))
-        return fail, norm, norm / norm0
-
 
 class ArmijoGoldsteinLS(NonlinearSolver):
     """
@@ -193,6 +181,7 @@ class ArmijoGoldsteinLS(NonlinearSolver):
 
         u = system._outputs
         du = system._vectors['output']['linear']
+        self.alpha = 1.
 
         self._run_apply()
         norm0 = self._iter_get_norm()
@@ -257,30 +246,21 @@ class ArmijoGoldsteinLS(NonlinearSolver):
         opt.declare('retry_on_analysis_error', default=True,
                     desc="Backtrack and retry if an AnalysisError is raised.")
 
-    def _iter_execute(self):
+    def _single_iteration(self):
         """
         Perform the operations in the iteration loop.
         """
         self._analysis_error_raised = False
         system = self._system
-        u = system._outputs
-        du = system._vectors['output']['linear']
 
         # Hybrid newton support.
         if self._do_subsolve and self._iter_count > 0:
-
             self._solver_info.append_solver()
 
             try:
                 cache = self._solver_info.save_cache()
-
-                for isub, subsys in enumerate(system._subsystems_allprocs):
-                    system._transfer('nonlinear', 'fwd', isub)
-
-                    if subsys in system._subsystems_myproc:
-                        subsys._solve_nonlinear()
-
-                system._apply_nonlinear()
+                self._gs_iter()
+                self._run_apply()
 
             except AnalysisError as err:
                 self._solver_info.restore_cache(cache)
@@ -295,48 +275,42 @@ class ArmijoGoldsteinLS(NonlinearSolver):
             finally:
                 self._solver_info.pop()
 
-        u.add_scal_vec(-self.alpha, du)
-        self.alpha *= self.options['rho']
-        u.add_scal_vec(self.alpha, du)
+        else:
+            self._run_apply()
 
-    def _run_iterator(self):
+    def _solve(self):
         """
         Run the iterative solver.
-
-        Returns
-        -------
-        boolean
-            Failure flag; True if failed to converge, False is successful.
-        float
-            absolute error.
-        float
-            relative error.
         """
         maxiter = self.options['maxiter']
         atol = self.options['atol']
         rtol = self.options['rtol']
         c = self.options['c']
 
+        system = self._system
+        u = system._outputs
+        du = system._vectors['output']['linear']
+
         self._iter_count = 0
         norm0, norm = self._iter_initialize()
         self._norm0 = norm0
-        self._mpi_print(self._iter_count, norm, norm / norm0)
 
         # Further backtracking if needed.
-        # The Armijo-Goldstein is basically a slope comparison --actual vs predicted.
-        # We don't have an actual gradient, but we have the Newton vector that should
-        # take us to zero, and our "runs" are the same, and we can just compare the
-        # "rise".
-        while self._iter_count < maxiter and (((norm0 - norm) < c * self.alpha * norm0) or
-                                              self._analysis_error_raised):
+        while (self._iter_count < maxiter and
+               ((norm > norm0 - c * self.alpha * norm0) or self._analysis_error_raised)):
             with Recording('ArmijoGoldsteinLS', self._iter_count, self) as rec:
-                self._iter_execute()
-                self._iter_count += 1
+
+                u.add_scal_vec(-self.alpha, du)
+                if self._iter_count > 0:
+                    self.alpha *= self.options['rho']
+                u.add_scal_vec(self.alpha, du)
+
+                cache = self._solver_info.save_cache()
+
                 try:
+                    self._single_iteration()
+                    self._iter_count += 1
 
-                    cache = self._solver_info.save_cache()
-
-                    self._run_apply()
                     norm = self._iter_get_norm()
 
                     # With solvers, we want to report the norm AFTER
@@ -347,6 +321,7 @@ class ArmijoGoldsteinLS(NonlinearSolver):
 
                 except AnalysisError as err:
                     self._solver_info.restore_cache(cache)
+                    self._iter_count += 1
 
                     if self.options['retry_on_analysis_error']:
                         self._analysis_error_raised = True
@@ -357,9 +332,5 @@ class ArmijoGoldsteinLS(NonlinearSolver):
                         exc = sys.exc_info()
                         reraise(*exc)
 
-            self._mpi_print(self._iter_count, norm, norm / norm0)
-
-        fail = (np.isinf(norm) or np.isnan(norm) or
-                (norm > atol and norm / norm0 > rtol))
-
-        return fail, norm, norm / norm0
+            # self._mpi_print(self._iter_count, norm, norm / norm0)
+            self._mpi_print(self._iter_count, norm, self.alpha)
