@@ -63,10 +63,10 @@ class NonlinearBlockGS(NonlinearSolver):
                              desc='lower limit for Aitken relaxation factor')
         self.options.declare('aitken_max_factor', default=1.5,
                              desc='upper limit for Aitken relaxation factor')
-        self.options.declare('cs_reconverge', default=True,
+        self.options.declare('cs_reconverge', types=bool, default=True,
                              desc='When True, when this driver solves under a complex step, nudge '
                              'the Solution vector by a small amount so that it reconverges.')
-        self.options.declare('use_apply_nonlinear', False,
+        self.options.declare('use_apply_nonlinear', types=bool, default=False,
                              desc="Set to True to always call apply_linear on the solver's system "
                              "after solve_nonlinear has been called.")
 
@@ -100,6 +100,7 @@ class NonlinearBlockGS(NonlinearSolver):
         """
         system = self._system
         outputs = system._outputs
+        residuals = system._residuals
         use_aitken = self.options['use_aitken']
 
         if use_aitken:
@@ -116,7 +117,11 @@ class NonlinearBlockGS(NonlinearSolver):
 
         if use_aitken or not self.options['use_apply_nonlinear']:
             # store a copy of the outputs
-            outputs_n = outputs._data.copy()
+            if not self.options['use_apply_nonlinear']:
+                with system._unscaled_context(outputs=[outputs]):
+                    outputs_n = outputs._data.copy()
+            else:
+                outputs_n = outputs._data.copy()
 
         self._solver_info.append_subsolver()
         self._gs_iter()
@@ -147,7 +152,11 @@ class NonlinearBlockGS(NonlinearSolver):
             else:
                 theta_n = 1.
 
-            outputs._data[:] = outputs_n
+            if not self.options['use_apply_nonlinear']:
+                with system._unscaled_context(outputs=[outputs]):
+                    outputs._data[:] = outputs_n
+            else:
+                outputs._data[:] = outputs_n
 
             # compute relaxed outputs
             outputs._data += theta_n * delta_outputs_n
@@ -157,7 +166,8 @@ class NonlinearBlockGS(NonlinearSolver):
 
         if not self.options['use_apply_nonlinear']:
             # Residual is the change in the outputs vector.
-            system._residuals._data[:] = outputs._data - outputs_n
+            with system._unscaled_context(outputs=[outputs], residuals=[residuals]):
+                residuals._data[:] = outputs._data - outputs_n
 
     def _run_apply(self):
         """
@@ -183,8 +193,10 @@ class NonlinearBlockGS(NonlinearSolver):
             # further increments the iteration counter.
             itercount += 1
             outputs = system._outputs
+            residuals = system._residuals
 
-            outputs_n = outputs._data.copy()
+            with system._unscaled_context(outputs=[outputs]):
+                outputs_n = outputs._data.copy()
 
             self._solver_info.append_subsolver()
             for isub, subsys in enumerate(system._subsystems_myproc):
@@ -193,7 +205,8 @@ class NonlinearBlockGS(NonlinearSolver):
                 system._check_reconf_update()
 
             self._solver_info.pop()
-            system._residuals._data[:] = outputs._data - outputs_n
+            with system._unscaled_context(residuals=[residuals]):
+                residuals._data[:] = outputs._data - outputs_n
 
     def _mpi_print_header(self):
         """
