@@ -5,6 +5,7 @@ from contextlib import contextmanager
 import os
 import re
 import sys
+import math
 import warnings
 import unittest
 from fnmatch import fnmatchcase
@@ -13,6 +14,7 @@ from six.moves import range, cStringIO as StringIO
 from collections import Iterable
 import numbers
 import json
+import importlib
 
 import numpy as np
 import openmdao
@@ -210,7 +212,7 @@ def ensure_compatible(name, value, shape=None, indices=None):
         if np.isscalar(value) or value.shape == (1,):
             value = np.ones(shape) * value
         else:
-            value = np.atleast_1d(value)
+            value = np.atleast_1d(value).astype(np.float64)
             if value.shape != shape:
                 raise ValueError("Incompatible shape for '%s': "
                                  "Expected %s but got %s." %
@@ -708,3 +710,54 @@ def json_loads_byteified(json_str):
         return _byteify(json.loads(json_str, object_hook=_byteify), ignore_dicts=True)
     else:
         return json.loads(json_str)
+
+
+_badtab = r'`~@#$%^&*()[]{}-+=|\/?<>,.:;'
+if PY2:
+    import string
+    _transtab = string.maketrans(_badtab, '_' * len(_badtab))
+else:
+    _transtab = str.maketrans(_badtab, '_' * len(_badtab))
+
+
+def str2valid_python_name(s):
+    """
+    Translate a given string into a valid python variable name.
+
+    Parameters
+    ----------
+    s : str
+        The string to be translated.
+
+    Returns
+    -------
+    str
+        The valid python name string.
+    """
+    return s.translate(_transtab)
+
+
+def make_serializable(o):
+    """
+    Recursively convert numpy types to native types for JSON serialization.
+
+    Parameters
+    ----------
+    o : object
+        the object to be converted
+
+    Returns
+    -------
+    object
+        The converted object.
+    """
+    if (isinstance(o, np.number)):
+        return o.item()
+    elif isinstance(o, np.ndarray):
+        return make_serializable(o.tolist())
+    elif isinstance(o, (list, tuple, set)):
+        return [make_serializable(item) for item in o]
+    elif '__dict__' in dir(o):
+        return make_serializable(o.__class__.__name__)
+    else:
+        return o
