@@ -236,6 +236,17 @@ class Vector(object):
         """
         return self.__iter__() if PY3 else list(self.__iter__())
 
+    def values(self):
+        """
+        Return values of variables contained in this vector.
+
+        Returns
+        -------
+        list
+            the variable values.
+        """
+        return [v for n, v in iteritems(self._views) if n in self._names]
+
     def __iter__(self):
         """
         Yield an iterator over variables involved in the current mat-vec product (relative names).
@@ -547,20 +558,7 @@ class Vector(object):
         """
         pass
 
-    def print_variables(self):
-        """
-        Print the names and values of all variables in this vector, one per line.
-        """
-        abs2prom = self._system._var_abs2prom[self._typ]
-        print('-' * 35)
-        print('   Vector %s, type %s' % (self._name, self._typ))
-        for abs_name, view in iteritems(self._views):
-            prom_name = abs2prom[abs_name]
-            print(' ' * 3, prom_name, view)
-        print('-' * 35)
-        print()
-
-    def set_complex_step_mode(self, active):
+    def set_complex_step_mode(self, active, keep_real=False):
         """
         Turn on or off complex stepping mode.
 
@@ -571,10 +569,17 @@ class Vector(object):
         ----------
         active : bool
             Complex mode flag; set to True prior to commencing complex step.
+
+        keep_real : bool
+            When this flag is True, keep the real value when turning off complex step. You only
+            need to do this when temporarily disabling complex step for guess_nonlinear.
         """
         if active:
             self._cplx_data[:] = self._data
 
+        elif keep_real:
+            self._cplx_data[:] = self._data.real
+
         self._data, self._cplx_data = self._cplx_data, self._data
         self._views, self._cplx_views = self._cplx_views, self._views
         self._views_flat, self._cplx_views_flat = self._cplx_views_flat, self._views_flat
